from sdcclient import SdSecureClient

from sdccli.time import duration_to_seconds


def retrieve_all_policy_events(secure: SdSecureClient, duration=None, scope=None, severity=None, type=None,
                               search=None, limit=None):
    if not duration:
        duration = "3D"

    if not limit:
        limit = 50

    duration_sec = duration_to_seconds(duration)
    if duration_sec is None:
        raise ValueError("The specified duration is incorrect")

    if duration_sec < 600:
        raise ValueError("The minimum duration is 10 minutes (10M)")

    filtering = []

    if scope:
        filtering.append(scope)

    if severity:
        number_severities = []
        sevs = set(sev.strip().lower() for sev in severity.split(","))

        if "high" in sevs:
            number_severities.extend([0, 1, 2, 3])
        if "med" in sevs:
            number_severities.extend([4, 5])
        if "low" in sevs:
            number_severities.append(6)
        if "info" in sevs:
            number_severities.append(7)

        text_severities = [f'"{sev}"' for sev in number_severities]

        severity_filter = f"severity in ({','.join(text_severities)})"
        filtering.append(severity_filter)

    if type:
        types_str = [f'"{t.strip().lower()}"'
                     for t in type.split(",")]
        type_filter = f"originator in ({','.join(types_str)})"
        filtering.append(type_filter)

    if search:
        search_filter = f'freeText in ("{search}")'
        filtering.append(search_filter)

    filter = " and ".join(filtering) if filtering else None
    all_events = []
    ok, res = secure.get_policy_events_duration(duration_sec=duration_sec, filter=filter)
    if not ok:
        raise Exception(res)

    all_events.extend(res["data"])

    while res["data"] and len(res["data"]) == res["ctx"]["limit"] and len(all_events) < limit:
        ok, res = secure.get_more_policy_events(res["ctx"])
        if not ok:
            raise Exception(res)
        all_events.extend(res["data"])

    return all_events[:limit]


def retrieve_policy_event_by_id(secure: SdSecureClient, event_id):
    ok, res = secure.get_policy_event(event_id)
    if not ok:
        raise Exception(res)

    return res



from sdccli.cli.formatter.text_formatter import alert, dashboard, panel, policy_events
from sdccli.cli.formatter.text_formatter.scanning import alert as scanning_alert
from sdccli.cli.formatter.text_formatter.scanning import image, vulnerability


class NormalFormatter:
    def __init__(self):
        self.formats = {}
        self.formats.update(alert.formats())
        self.formats.update(dashboard.formats())
        self.formats.update(scanning_alert.formats())
        self.formats.update(image.formats())
        self.formats.update(panel.formats())
        self.formats.update(vulnerability.formats())
        self.formats.update(policy_events.formats())

    def format(self, response, format_type):
        if format_type in self.formats:
            self.formats[format_type](response)
        else:
            print(response)



import click
import json
import sys
from datetime import datetime
from sdccli.printer import print_list, print_item
from prettytable import PrettyTable, PLAIN_COLUMNS


_item_keys = ["id", "name", "schedule", "scope", "moduleName", "enabled"]
_list_keys = _item_keys


@click.group(name='compliance', short_help='Sysdig Secure compliance operations')
def compliance():
    pass


@compliance.command(name='add', short_help="Add a compliance task")
@click.argument('name', nargs=1)
@click.option('--scope', nargs=1, help="The agent will only run the task on hosts matching this scope or on hosts where containers match this scope.")
@click.option('--schedule', nargs=1, default="06:00:00Z/PT12H", help="The frequency at which this task should run. Expressed as an ISO 8601 Duration <https://en.wikipedia.org/wiki/ISO_8601#Durations>")
@click.option('--module', nargs=1, default="docker-bench-security", help="The name of the module that implements this task. Separate from task name in case you want to use the same module to run separate tasks with different scopes or schedules. [ 'docker-bench-security', 'kube-bench' ]")
@click.option('--disabled', nargs=1, is_flag=True, help="Disable this task to don't run as defined by its schedule.")
@click.pass_obj
def add(cnf, name, scope, schedule, module, disabled):
    """
    NAME: The name of the task e.g. 'Check Docker Compliance'.
    """
    ok, res = cnf.sdsecure.add_compliance_task(
        name,
        module_name=module,
        schedule=schedule,
        scope=scope,
        enabled=not disabled)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res, _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@compliance.command(name='list', short_help="List compliance tasks")
@click.pass_obj
def list(cnf):
    ok, res = cnf.sdsecure.list_compliance_tasks()

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_list(res, _list_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@compliance.command(name='results', short_help="Get the list of compliance results")
@click.argument('filter', nargs=1, required=False)
@click.option('--limit', nargs=1, default=50, help='Maximum number of compliacne results to list. If is not provided it will list up to 50 results.')
@click.pass_obj
def results(cnf, filter, limit):
    """
    FILTER: an optional case insensitive filter used to match against the completed task name and return matching results.
    """
    if filter is None:
        filter = ""
    ok, res = cnf.sdsecure.list_compliance_results(filter=filter, limit=limit)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        _print_results(res["results"])
    else:
        print("Error: " + str(res))
        sys.exit(1)


@compliance.command(name='result', short_help="Get the result of one compliance run")
@click.argument('result-id', nargs=1)
@click.option('--section', nargs=1, type=click.INT, help="Display only the tests in the selected seccion (1-7)")
@click.option('--level', nargs=1, default="warn", help="Display only the tests with status level higher or equal to the provided one. Valid levels are: [fail, warn, pass, info]")
@click.option('--remediation', is_flag=True, help="Display the information on how to remediate the issue")
@click.option('--csv', is_flag=True, help="Get result-id in csv format")
@click.pass_obj
def result(cnf, result_id, section, level, remediation, csv):
    """
    RESULT-ID: id of a compliance result to display
    """
    if csv:
        ok, res = cnf.sdsecure.get_compliance_results_csv(result_id)
        if ok:
            print(res)
            return
    else:
        ok, res = cnf.sdsecure.get_compliance_results(result_id)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        _print_result(res, section, level, remediation)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@compliance.command(name='get', short_help="Get a compliance task")
@click.argument('task', nargs=1, type=click.INT)
@click.pass_obj
def get(cnf, task):
    """
    TASK: compliance task id to get
    """
    ok, res = cnf.sdsecure.get_compliance_task(task)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res, _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@compliance.command(name='update', short_help="Update a compliance task")
@click.argument('task', nargs=1, type=click.INT)
@click.option('--name', nargs=1, help="The name of the task e.g. 'Check Docker Compliance'.")
@click.option('--scope', nargs=1, help="The agent will only run the task on hosts matching this scope or on hosts where containers match this scope.")
@click.option('--schedule', nargs=1, default="06:00:00Z/PT12H", help="The frequency at which this task should run. Expressed as an ISO 8601 Duration <https://en.wikipedia.org/wiki/ISO_8601#Durations>")
@click.option('--module', nargs=1, default="docker-bench-security", help="The name of the module that implements this task. Separate from task name in case you want to use the same module to run separate tasks with different scopes or schedules. [ 'docker-bench-security', 'kube-bench' ]")
@click.option('--disabled', nargs=1, is_flag=True, help="Disable this task to don't run as defined by its schedule.")
@click.pass_obj
def update(cnf, task, name, scope, schedule, module, disabled):
    """
    TASK: compliance task id to update
    """
    ok, res = cnf.sdsecure.update_compliance_task(
        task,
        name=name,
        module_name=module,
        schedule=schedule,
        scope=scope,
        enabled=not disabled)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res, _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@compliance.command(name='del', short_help="Delete compliance tasks")
@click.argument('tasks', nargs=-1, type=click.INT, required=True)
@click.pass_obj
def delete(cnf, tasks):
    """
    TASKS: compliance task ids to delete
    """
    for task in tasks:
        ok, res = cnf.sdsecure.delete_compliance_task(task)

        if not ok:
            print("Error: " + str(res))
            sys.exit(1)

    print("Success")


def _print_result(result, section, level, remediation):
    print_item(result, ["id", "name", "risk", "hostMac"])
    print("{:25} {}/{}/{}".format("Tests fail/warn/pass:", result["failCount"], result["warnCount"], result["passCount"]))
    date = datetime.utcfromtimestamp(result["timestampNs"] / 10**9)
    print("{:25} {}".format("Completed on:", date))

    levels = ["fail", "warn", "pass", "info"]
    valid_levels = levels[:levels.index(level) + 1]
    for test in result["tests"]:
        if section and int(test["sectionId"]) != section:
            continue

        test_results = [r for r in test["results"] if r["status"] in valid_levels]
        if not test_results:
            continue

        print("\n{}. {}".format(test["sectionId"], test["description"]))
        for result in test_results:
            print("{:4} {} - {}".format(result["testNumber"], result["status"], result["description"]))
            if result["items"]:
                print("{}{} {}".format(" " * 5, result["details"], result["items"]))
            if remediation and result["remediation"]:
                print("{}{}".format(" " * 5, result["remediation"]))


def _print_results(results):
    t = PrettyTable(["id", "risk", "name", "host mac", "date", "tests"])
    t.set_style(PLAIN_COLUMNS)
    t.align = 'l'

    for result in results:
        values = []
        values.append(result["id"])
        values.append(result["risk"])
        values.append(result["taskName"])
        values.append(result["hostMac"])
        date = datetime.utcfromtimestamp(result["timestampNs"] / 10**9)
        values.append(date)
        tests = "{}/{}".format(result["passCount"], result["testsRun"])
        values.append(tests)
        t.add_row(values)
    print(t.get_string())



import sys

import click
import dateutil.parser

from sdccli import helpers
from sdccli.usecases.scanning import image as use_case


@click.group(short_help='Image operations')
def image():
    pass


@image.command(name='add', short_help="Add an image")
@click.argument('input_image', nargs=1)
@click.option('--force', is_flag=True, help="Force reanalysis of image")
@click.option('--dockerfile', type=click.Path(exists=True), metavar='<Dockerfile>',
              help="Submit image's dockerfile for analysis")
@click.option('--annotation', nargs=1, multiple=True)
@click.option('--noautosubscribe', is_flag=True,
              help="If set, instruct the engine to disable tag_update subscription for the added tag.")
@click.pass_obj
def add(cnf, input_image, force, dockerfile, annotation, noautosubscribe):
    """
    INPUT_IMAGE: Input image can be in the following formats: registry/repo:tag
    """
    dockerfile_contents = None
    if dockerfile:
        try:
            with open(dockerfile) as f:
                dockerfile_contents = f.read()
        except Exception as error:
            print("Error parsing dockerfile file (%s): %s" % (dockerfile, str(error)))
            sys.exit(1)

    annotations = helpers.annotation_arguments_to_map(annotation)
    annotations.update({"added-by": "sysdig-cli"})

    try:
        res = use_case.add_scanning_image(cnf.sdscanning,
                                          input_image,
                                          force=force,
                                          dockerfile=dockerfile_contents,
                                          annotations=annotations,
                                          autosubscribe=not noautosubscribe)
        cnf.formatter.format(res, "scanningImageAdded")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@image.command(name='list', short_help="List all images")
@click.option('--full', is_flag=True, help="Show full row output for each image")
@click.option('--show-all', is_flag=True,
              help="Show all images in the system instead of just the latest for a given tag")
@click.pass_obj
def imagelist(cnf, full, show_all):
    try:
        res = use_case.list_scanning_images(cnf.sdscanning, show_all=show_all)
        cnf.formatter.format(res, "scanningImageList")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@image.command(name='get', short_help="Get an image")
@click.argument('input_image', nargs=1)
@click.option('--show-history', is_flag=True,
              help="Show history of images that match the input image, if input image is of the form registry/repo:tag")
@click.pass_obj
def get(cnf, input_image, show_history):
    """
    INPUT_IMAGE: Input image can be in the following formats: Image Digest, ImageID or registry/repo:tag
    """
    try:
        res = use_case.get_image_from_digest_id_or_repo(cnf.sdscanning, input_image, show_history)
        cnf.formatter.format(res, "scanningImage")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@image.command(name='content', short_help="Get contents of image")
@click.argument('input_image', nargs=1)
@click.argument('content_type', nargs=1, required=True)
@click.pass_obj
def query_content(cnf, input_image, content_type):
    """
    INPUT_IMAGE: Input image can be in the following formats: Image Digest, ImageID or registry/repo:tag

    CONTENT_TYPE: The content type can be one of the following types:

      - os: Operating System Packages

      - files: Files

      - npm: Node.JS NPM Module

      - gem: Ruby GEM

      - python: Python modules

      - java: Java packages

      - all: includes [os, npm, gem, python, java] contents

      - all+files: includes [os, npm, gem, python, java, files] contents
    """
    try:
        res = use_case.query_image_content(cnf.sdscanning, input_image, content_type)
        cnf.formatter.format((res, content_type), "scanningQueryImage")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@image.command(name='metadata', short_help="Get metadata about an image")
@click.argument('input_image', nargs=1)
@click.argument('metadata_type', nargs=1, required=False)
@click.pass_obj
def query_metadata(cnf, input_image, metadata_type):
    """
    INPUT_IMAGE: Input image can be in the following formats: Image Digest, ImageID or registry/repo:tag

    METADATA_TYPE: The metadata type can be one of the types returned by running without a type specified
    """
    try:
        res = use_case.query_image_metadata(cnf.sdscanning, input_image, metadata_type)
        cnf.formatter.format((res, metadata_type), "scanningQueryImage")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@image.command(name='evaluation', short_help="Check latest policy evaluation for an image")
@click.option("--detail", is_flag=True, help="Show detailed policy evaluation report")
@click.option("--policy", help="Specify which POLICY to use for evaluate (defaults currently active policy)")
@click.argument('input_image', nargs=1)
@click.pass_obj
def check(cnf, input_image, detail, policy):
    """
    INPUT_IMAGE: Input image can be in the following formats: Image Digest, ImageID or registry/repo:tag
    """
    try:
        res = use_case.get_image_scanning_results(cnf.sdscanning, input_image, policy=policy)
        cnf.formatter.format((res, detail), "scanningEvaluationImage")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        if policy:
            print("Check if the image has been assigned to the provided policy")
        sys.exit(1)


@image.command(name='vuln', short_help="Get image vulnerabilities")
@click.argument('input_image', nargs=1)
@click.argument('vuln_type', nargs=1, default='all')
@click.option('--vendor-only', default=True, type=bool,
              help="Show only vulnerabilities marked by upstream vendor as applicable (default=True)")
@click.pass_obj
def query_vuln(cnf, input_image, vuln_type, vendor_only):
    """
    INPUT_IMAGE: Input image can be in the following formats: Image Digest, ImageID or registry/repo:tag

    VULN_TYPE: VULN_TYPE: Vulnerability type can be one of the following types:

      - os: CVE/distro vulnerabilities against operating system packages

      - non-os: NPM, Gem, Java Archive (jar, ear, war) and Python PIP CVEs

      - all: combination report containing both 'os' and 'non-os' vulnerability records (default)
    """
    try:
        res = use_case.query_image_vuln(cnf.sdscanning, input_image, vuln_type, vendor_only=vendor_only)
        cnf.formatter.format((res, vuln_type), "scanningVulnImage")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@image.command(name='pdf-report', short_help="Get the scan report for an image in pdf")
@click.option("--tag", help="Specify which TAG is evaluated for a given image ID or Image Digest")
@click.option("--date", help="Specify a date for the report")
@click.argument('input_image', nargs=1)
@click.argument('pdf_path', nargs=1)
@click.pass_obj
def pdf_report(cnf, input_image, pdf_path, tag, date):
    """
    INPUT_IMAGE: Input image can be in the following formats: Image Digest, ImageID or registry/repo:tag

    PDF_PATH: The name of the pdf that will be generated with the report
    """
    try:
        res = use_case.get_pdf_report(
            cnf.sdscanning,
            input_image,
            tag=tag,
            date=dateutil.parser.parse(date).strftime("%Y-%m-%dT%XZ") if date else None
        )
        with open(pdf_path, 'wb') as f:
            f.write(res)
        print("PDF %s saved" % pdf_path)

    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@image.command(name='del', short_help="Delete an image")
@click.argument('input_image', nargs=-1, required=True)
@click.option('--force', is_flag=True,
              help="Force deletion of image by cancelling any subscription/notification settings prior to image delete")
@click.pass_obj
def delete(cnf, input_image, force):
    """
    INPUT_IMAGE: Input image can be in the following formats: Image Digest, ImageID or registry/repo:tag
    """
    for image in input_image:
        ok, res = cnf.sdscanning.delete_image(image, force=force)

        if not ok:
            print("Error: " + str(res))
            sys.exit(1)

    print("Success")



import json

import requests
from sdcclient import SdSecureClient, SdMonitorClient
from sdcclient._common import _SdcCommon

from sdccli.falco_macro import SortMacrosByDepedency

ORIGIN_SECURE_UI = "Secure UI"


def dashboards(file_data, sdmonitor, existing_dashboards, restore_unowned):
    errors = []
    ok, userdata = sdmonitor.get_user_info()
    if not ok:
        return False, userdata

    username = userdata["user"]["username"]
    dashboards = [dboard for dboard in file_data if
                  restore_unowned or "username" in dboard and username in dboard["username"]]
    total = len(dashboards)
    for i, dashboard in enumerate(dashboards):
        name = dashboard['name']
        print("[{}/{}] Creating dashboard {}... ".format(i + 1, total, name.strip()))
        if name in existing_dashboards:
            if same_dict(dashboard, existing_dashboards[name],
                         ['id', 'customerId', 'teamId',
                          'userId', 'username', 'autoCreated', 'publicToken']):
                print("      already exists, skip")
                continue
            else:
                sdmonitor.delete_dashboard(existing_dashboards[name])

        dashboard['id'] = None
        dashboard['version'] = None

        # We need to update the team IDs in case the old ones have been removed
        if dashboard.get("teamName") is not None:
            ok, res = sdmonitor.get_team(dashboard.get("teamName"))
            if ok:
                dashboard["teamId"] = res["id"]
        if "sharingSettings" in dashboard:
            for setting in dashboard["sharingSettings"]:
                if setting["member"]["name"] is not None:
                    ok, res = sdmonitor.get_team(setting["member"]["name"])
                    if ok:
                        setting["member"]["id"] = res["id"]

        ok, result = sdmonitor.create_dashboard_with_configuration(dashboard)
        if not ok:
            errors.append("couldn't recreate dashboard " + name + ': ' + result)

    if len(errors) != 0:
        return False, errors
    return True, None


def alerts(file_data, monitor: SdMonitorClient):
    errors = []
    total = len(file_data)
    ok, res = monitor.get_alerts()
    if not ok:
        return False, res
    existing_alerts = {alert['name']: alert for alert in res['alerts']}

    ok, res = monitor.list_notification_channels()
    if not ok:
        return False, res
    existing_notification_channels = {nc['name']: nc['id'] for nc in res['notificationChannels']}

    for i, alert in enumerate(file_data, start=1):
        name = alert['name']
        alert['notificationChannelIds'] = []
        for nc in alert.get('notificationChannelNames', []):
            if nc in existing_notification_channels:
                alert['notificationChannelIds'].append(existing_notification_channels[nc])
            else:
                print(f"  Warning: The Notification Channel '{nc}' does not exist in the environment, "
                      f"the restored alert won't be assigned to this NC.")
        print(f"[{i}/{total}] Creating alert {name}... ")
        if name in existing_alerts:
            if same_dict(alert, existing_alerts[name],
                         ['id', 'teamId', 'autoCreated',
                          'notificationChannelNames', 'notificationCount', 'invalidMetrics',
                          'valid']):
                print("      already exists, skip")
                continue
            else:
                ok, res = monitor.delete_alert(existing_alerts[name])
                if not ok:
                    errors.append(res)
                    continue

        ok, result = monitor.create_alert(alert_obj=alert)
        if not ok:
            errors.append(result)

    if len(errors) > 0:
        return False, errors
    return True, None


def users(sdcommon, filename):
    '''
    Restores users without dropping the existing ones
    '''
    with open(filename) as file:
        file_data = json.load(file)

        existing_user_list = []
        ok, data = sdcommon.get_users()
        if not ok:
            return False, data

        existing_user_list = [user["username"] for user in data]

        errors = []
        users = [user for user in file_data if not user["username"] in existing_user_list]
        total = len(users)
        for i, user in enumerate(users):
            print("[{}/{}] Creating user {}... ".format(i + 1, total, user['username']))
            ok, res = sdcommon.create_user_invite(user["username"], user.get("firstName", ""), user.get("lastName", ""),
                                                  user["systemRole"])
            if not ok:
                errors.append("couldn't recreate user " + user["username"] + " : " + res)

        if len(errors) != 0:
            return False, errors
        return True, None


# TODO: Move this to the python-sdc-client
def get_user_created_rules(secure: SdSecureClient):
    ok, res = secure.list_rules()
    if not ok:
        return False, res

    user_created_rules = []
    for list in res:
        for j, id in enumerate(list['ids']):
            if list['publishedBys'][j]['origin'] == ORIGIN_SECURE_UI:
                user_rules = list.copy()

                user_rules['id'] = list['ids'][j]
                del user_rules['ids']

                user_rules['publishedBy'] = list['publishedBys'][j]
                del user_rules['publishedBys']

                user_created_rules.append(user_rules)

    return True, user_created_rules


def rules(file_data, sdsecure: SdSecureClient):
    errors = []
    total = len(file_data)
    ok, existing_rules = get_user_created_rules(sdsecure)
    if not ok:
        return False, existing_rules

    existing_rules = {l['name']: l for l in existing_rules}
    for i, rule in enumerate(file_data, start=1):
        name = rule["name"]
        print(f"[{i}/{total}] Creating rule {name}... ")
        if name in existing_rules:
            ok, erule = sdsecure.get_rule_id(existing_rules[name]['id'])
            if not ok:
                errors.append(erule)
                continue

            if same_dict(rule, erule,
                         ['id', 'filename', 'origin', 'sysdig',
                          'versionId']):
                print("      already exists, skip")
                continue

            ok, res = sdsecure.update_rule(erule['id'], rule['details'], rule['description'], rule['tags'])
            if not ok:
                errors.append(res)
            continue

        ok, result = sdsecure.add_rule(name, rule["details"], rule["description"], rule["tags"])
        if not ok:
            errors.append("couldn't recreate rule " + name + ': ' + result)

    if len(errors) > 0:
        return False, errors
    return True, None


# TODO: Move this to the python-sdc-client
def get_user_created_lists(secure: SdSecureClient):
    ok, res = secure.list_falco_lists()
    if not ok:
        return False, res

    user_created_lists = []
    for list in res:
        for j, id in enumerate(list['ids']):
            if list['publishedBys'][j]['origin'] == ORIGIN_SECURE_UI:
                user_list = list.copy()

                user_list['id'] = list['ids'][j]
                del user_list['ids']

                user_list['publishedBy'] = list['publishedBys'][j]
                del user_list['publishedBys']

                user_created_lists.append(user_list)

    return True, user_created_lists


# FIXME: Remove this as soon as the PR is merged in the python-sdc-client
def _add_falco_list(self, name, items, append=False):
    '''**Description**
        Create a new list

    **Arguments**
        - name: A name for this object. Should exactly be the value of the "list" property of the yaml object.
        - items: the array of items as represented in the yaml List.

    **Success Return Value**
        A JSON object representing the falco list.
    '''
    flist = {
        "name": name,
        "items": {
            "items": items
        },
        "append": append,
    }
    res = self.http.post(self.url + '/api/secure/falco/lists', data=json.dumps(flist), headers=self.hdrs,
                         verify=self.ssl_verify)
    return self._request_result(res)


def falco_lists(file_data, secure: SdSecureClient):
    errors = []
    total = len(file_data)
    ok, existing_lists = get_user_created_lists(secure)
    if not ok:
        return False, existing_lists
    existing_lists = {l['name']: l for l in existing_lists}
    for i, falco_list in enumerate(file_data, start=1):
        name = falco_list["name"]
        print(f"[{i}/{total}] Creating falco list {name}... ")
        if name in existing_lists:
            ok, elist = secure.get_falco_list_id(existing_lists[name]['id'])
            if not ok:
                errors.append(elist)
                continue

            if same_dict(
                    falco_list, elist,
                    ['id', 'filename', 'origin', 'sysdig', 'versionId']) and elist['items']['items'] == \
                    falco_list['items']['items']:
                print("      already exists, skip")
                continue

            ok, res = secure.update_falco_list(elist['id'], falco_list['items']['items'])
            if not ok:
                errors.append(res)
            continue

        ok, result = _add_falco_list(secure, name=name, items=falco_list['items']['items'], append=falco_list['append'])
        if not ok:
            errors.append(f"couldn't recreate falco list {name}: {result}")

    if len(errors) > 0:
        return False, errors
    return True, None


# TODO: Move this method to the python-sdc-client
def get_user_created_macros(secure: SdSecureClient):
    ok, res = secure.list_falco_macros()
    if not ok:
        return False, res

    user_created_macros = []
    for macro in res:
        for j, id in enumerate(macro['ids']):
            if macro['publishedBys'][j]['origin'] == ORIGIN_SECURE_UI:
                user_macro = macro.copy()

                user_macro['id'] = macro['ids'][j]
                del user_macro['ids']

                user_macro['publishedBy'] = macro['publishedBys'][j]
                del user_macro['publishedBys']

                user_created_macros.append(user_macro)

    return True, user_created_macros


# FIXME: Please, avoid evil hacks and send a PR
def _add_falco_macro(self, name, condition, append=False):
    '''**Description**
        Create a new macro

    **Arguments**
        - name: A name for this object. Should exactly be the value of the "macro" property of the yaml object.
        - condition: the full condition text exactly as represented in the yaml file.

    **Success Return Value**
        A JSON object representing the falco macro.
    '''
    macro = {
        "name": name,
        "condition": {
            "components": [],
            "condition": condition
        },
        "append": append
    }
    res = requests.post(self.url + '/api/secure/falco/macros', data=json.dumps(macro), headers=self.hdrs,
                        verify=self.ssl_verify)
    return self._request_result(res)


def falco_macros(file_data, sdsecure: SdSecureClient):
    # Falco macros must be created in order of dependency
    file_data = SortMacrosByDepedency(file_data).sort(reverse=True)
    errors = []
    total = len(file_data)
    ok, existing_macros = get_user_created_macros(sdsecure)

    existing_macros = {macro["name"]: macro for macro in existing_macros}
    for i, macro in enumerate(file_data, start=1):
        name = macro["name"]
        print(f"[{i}/{total}] Creating falco macro {name}... ")
        if name in existing_macros:
            ok, emacro = sdsecure.get_falco_macro_id(existing_macros[name]['id'])
            if not ok:
                errors.append(emacro)
                continue
            if same_dict(
                    macro, emacro,
                    ['id', 'filename', 'origin', 'sysdig', 'versionId']) and \
                    emacro['condition']['condition'] == macro['condition']['condition']:
                print("      already exists, skip")
                continue

            sdsecure.update_falco_macro(emacro['id'], macro['condition']['condition'])
            continue

        ok, result = _add_falco_macro(sdsecure, name=name, condition=macro["condition"]["condition"],
                                      append=macro['append'])
        if not ok:
            errors.append("couldn't recreate falco macro " + name + ': ' + result)

    if len(errors) != 0:
        return False, errors
    return True, None


def policies(file_data, secure: SdSecureClient):
    errors = []

    ok, existing_policies = secure.list_policies()
    if not ok:
        return False, existing_policies

    existing_policies = {policy['name']: policy for policy in existing_policies}
    total = len(file_data)
    for i, policy in enumerate(file_data, start=1):
        name = policy['name']
        print(f"[{i}/{total}] Creating policy {name}... ")
        if name in existing_policies:
            if same_dict(policy, existing_policies[name],
                         ['name', 'description', 'ruleNames', 'actions', 'notificationChannelIds', 'origin',
                          'versionId']):
                print("      already exists, skip")
                continue
            else:
                ok, result = secure.update_policy(
                    id=existing_policies[name]['id'],
                    name=policy['name'],
                    description=policy['description'],
                    rule_names=policy['ruleNames'],
                    actions=policy['actions'],
                    scope=policy['scope'] if 'scope' in policy else None,
                    severity=policy['severity'],
                    enabled=policy['enabled'],
                    notification_channels=policy['notificationChannelIds']
                )
                if not ok:
                    errors.append(result)
                continue

        ok, result = secure.add_policy(
            name,
            policy['description'],
            rule_names=policy['ruleNames'],
            actions=policy['actions'],
            scope=policy['scope'] if 'scope' in policy else None,
            severity=policy['severity'],
            enabled=policy['enabled'],
            notification_channels=policy['notificationChannelIds'],
            type=policy.get('type', 'falco')
        )
        if not ok:
            errors.append(result)

    if len(errors) > 0:
        return False, errors
    return True, None


# TODO: Remove this, we do not support priorities anymore
def policy_priorities(sdsecure, filename, ids_changed=None):
    with open(filename) as file:
        file_data = json.load(file)
        del file_data["priorities"]["createdOn"]
        del file_data["priorities"]["modifiedOn"]

        errors = []

        if ids_changed is not None:
            oldIdsChanged = set([int(key) for key in ids_changed.keys()])
            priorityList = file_data["priorities"]["policyIds"]
            for i, oldId in enumerate(priorityList):
                if oldId in oldIdsChanged:
                    priorityList[i] = int(ids_changed[str(oldId)])

            file_data["priorities"]["policyIds"] = priorityList

        print("Retrieving current version of the priorities... ")
        ok, res = sdsecure.get_policy_priorities()
        if not ok:
            errors.append("couldn't retrieve the current version of the priorities: " + res)
            return False, errors  # without the current version we can't restore the priorities
        else:
            current_version = res["priorities"]["version"]
            file_data["priorities"]["version"] = current_version

        print("Restoring policy priorities... ")
        ok, result = sdsecure.set_policy_priorities(json.dumps(file_data))
        if not ok:
            errors.append("couldn't restore policy priorities: " + result)

        if len(errors) != 0:
            return False, errors
        else:
            return True, None


def teams(file_data, sdcommon: _SdcCommon):
    errors = []
    total = len(file_data)

    okTeams, res = sdcommon.get_teams()
    if not okTeams:
        return False, res

    existing_teams = {team['name']: team for team in res}
    mutable_teams = [team for team in file_data if not team['immutable']]
    total = len(mutable_teams)
    for i, team in enumerate(mutable_teams, start=1):
        print(f"[{i}/{total}] Creating team {team['name'].strip()}... ")
        memberships = {member['userName']: member['role'] for member in team['userRoles']}
        filter = team['filter'] if 'filter' in team else ''
        description = team['description'] if 'description' in team else ''
        if team['name'] in existing_teams:
            ok, result = sdcommon.edit_team(name=team['name'],
                                            memberships=memberships,
                                            filter=filter,
                                            description=description,
                                            show=team['show'],
                                            theme=team['theme'],
                                            perm_capture=team['canUseSysdigCapture'],
                                            perm_custom_events=team['canUseCustomEvents'],
                                            perm_aws_data=team['canUseAwsMetrics'])
        else:
            ok, result = sdcommon.create_team(name=team['name'],
                                              memberships=memberships,
                                              filter=filter,
                                              description=description,
                                              show=team['show'],
                                              theme=team['theme'],
                                              perm_capture=team['canUseSysdigCapture'],
                                              perm_custom_events=team['canUseCustomEvents'],
                                              perm_aws_data=team['canUseAwsMetrics'])

        if not ok:
            if "409" in result:  # Team already existing, not an error
                print("Team " + team['name'] + " already exists")
                continue
            errors.append("couldn't recreate team " + team['name'] + ': ' + result)

    if len(errors) != 0:
        return False, errors
    return True, None


def notification_channels(file_data, sdmonitor, existing_channels):
    errors = []
    ids_changed = {}
    total = len(file_data)
    for i, channel in enumerate(file_data):
        id = channel["id"]
        name = channel["name"] if "name" in channel else ""
        type = channel["type"] if "type" in channel else ""
        print("[{}/{}] Creating notification channel {} of type {}... ".format(i + 1, total, name, type))
        if name in existing_channels:
            if same_dict(channel, existing_channels[name],
                         ['id']):
                print("      already exists, skip")
                ids_changed[str(id)] = str(existing_channels[name]['id'])
                continue
            else:
                sdmonitor.delete_notification_channel(existing_channels[name])

        del channel["teamId"]
        ok, result = sdmonitor.create_notification_channel(channel)
        if not ok:
            errors.append("couldn't recreate team " + channel['name'] + ': ' + result)
        else:
            ids_changed[str(id)] = str(result["notificationChannel"]["id"])

    if errors:
        return False, errors
    return True, ids_changed


def user_falco_rules(sdsecure, filename):
    with open(filename) as file:
        file_data = json.load(file)
        print('Restoring user falco rules...')
        ok, result = sdsecure.set_user_falco_rules(file_data["userRulesFile"]["content"])
        if not ok:
            return False, result
        return True, None


def same_dict(d1, d2, skip_keys):
    if (d1.keys() - skip_keys) != (d2.keys() - skip_keys):
        return False

    for k in d1.keys():
        if k in skip_keys:
            continue
        if d1[k] != d2[k]:
            return False

    return True



from multiprocessing.pool import ThreadPool

from sdcclient import SdScanningClient


def get_vulnerability_details(client: SdScanningClient, id):
    ok, res = client.get_vulnerability_details(id)
    if not ok:
        raise Exception(res)

    return res


def add_vulnerability_exception_bundle(client: SdScanningClient, name, comment):
    ok, res = client.add_vulnerability_exception_bundle(name=name, comment=comment)
    if not ok:
        if res == "Conflict":
            res = "This bundle already exists"
        raise Exception(res)

    return res


def list_vulnerability_exception_bundles(client: SdScanningClient):
    ok, res = client.list_vulnerability_exception_bundles()
    if not ok:
        raise Exception(res)

    pool = ThreadPool()
    res = pool.map(lambda b: get_vulnerability_exception_bundle(client, b["id"]), res)

    return res


def add_vulnerability_exception(client: SdScanningClient, bundle, cve, note, expiration_date):
    ok, res = client.add_vulnerability_exception(bundle, cve, note, expiration_date)
    if not ok:
        raise Exception(res)

    return res


def delete_vulnerability_exception(client: SdScanningClient, bundle, cve):
    ok, res = client.get_vulnerability_exception_bundle(bundle)
    if not ok:
        raise Exception(res)

    found_cves = [c for c in res["items"] if c["trigger_id"] == cve]
    if len(found_cves) == 0:
        raise Exception(f"CVE not found in bundle {bundle}")

    ok, res = client.delete_vulnerability_exception(bundle, found_cves[0]["id"])
    if not ok:
        raise Exception(res)

    return res


def update_vulnerability_exception(client: SdScanningClient, bundle, id, cve=None, enabled=None, note="__unset__",
                                   expiration_date="__unset__"):
    ok, res = client.get_vulnerability_exception_bundle(bundle)
    if not ok:
        raise Exception(res)

    vulns_found = [vuln for vuln in res["items"] if vuln["id"] == id]
    if len(vulns_found) == 0:
        raise Exception(f"Vulnerability exception not found for bundle {res['name']} with ID: {id}")

    vuln = vulns_found[0]
    if cve is None:
        cve = vuln["trigger_id"]
    if enabled is None:
        enabled = vuln["enabled"]
    if note == "__unset__":
        note = vuln["notes"]
    if expiration_date == "__unset__":
        expiration_date = vuln["expiration_date"]

    ok, res = client.update_vulnerability_exception(bundle, id, cve, enabled, note, expiration_date)
    if not ok:
        raise Exception(res)

    return res


def get_vulnerability_exception_bundle(client: SdScanningClient, bundle):
    ok, res = client.get_vulnerability_exception_bundle(bundle)
    if not ok:
        raise Exception(res)
    return res


def delete_vulnerability_exception_bundle(client: SdScanningClient, bundle):
    ok, res = client.delete_vulnerability_exception_bundle(bundle)
    if not ok:
        raise Exception(res)
    return res


def download_vulnerability_report(client: SdScanningClient, runtime, os, non_os, out_file):
    scope = "runtime" if runtime else "static"

    header = ""
    os_data = ""
    non_os_data = ""

    if (not os and not non_os) or os:
        ok, res = client.download_cve_report_csv(vuln_type="os", scope_type=scope)
        if not ok:
            raise Exception(res)
        os_data = "\n".join(res.splitlines()[1:]) + "\n"
        header = res.splitlines()[0] + "\n"

    if (not os and not non_os) or non_os:
        ok, res = client.download_cve_report_csv(vuln_type="non-os", scope_type=scope)
        if not ok:
            raise Exception(res)
        non_os_data = "\n".join(res.splitlines()[1:]) + "\n"
        header = res.splitlines()[0] + "\n"

    out_file.write(header)
    out_file.write(os_data)
    out_file.write(non_os_data)



def alert_add(monitor, name, description, severity, atleast, condition, disable, segment, segment_condition,
              user_filter, notify, annotation, type):
    annotations = {}
    if annotation:
        for a in annotation:
            try:
                (k, v) = a.split('=', 1)
                if k and v:
                    annotations[k] = v
                else:
                    return [False, "Error: found null in key or value"]
            except Exception:
                return [False,
                        f"Error: annotation format error - annotations must be of the form (--annotation key=value), "
                        "found: {a}"]

    ok, res = monitor.create_alert(
        name=name,
        description=description,
        severity=severity,
        for_atleast_s=atleast,
        condition=condition,
        segmentby=segment if segment else [],
        segment_condition=segment_condition,
        user_filter=user_filter,
        notify=notify,
        enabled=not disable,
        annotations=annotations,
        type=type)

    if not ok:
        raise Exception(res)

    return res['alert']


def alert_get(monitor, alert):
    alert_id = None
    try:
        alert_id = int(alert)
    except ValueError:
        pass

    ok, res = monitor.get_alerts()
    if not ok:
        raise Exception(res)

    for alert_itr in res['alerts']:
        if alert == alert_itr['name'] or (alert_id is not None and alert_id == alert_itr['id']):
            return alert_itr

    raise Exception(f"no alert with id {alert_id}"
                    if alert_id is not None else
                    f"no alert with name {alert}")


def alert_update(monitor, alert):
    ok, res = monitor.update_alert(alert)
    if not ok:
        raise Exception(res)
    return res['alert']


def alert_list(monitor):
    ok, res = monitor.get_alerts()
    if not ok:
        raise Exception(res)

    return res['alerts']


def alert_delete(monitor, alerts):
    removed_alerts = []
    for alert_id in alerts:
        ok, res = monitor.delete_alert({"id": alert_id})
        removed_alerts.append({"id": alert_id})
        if not ok:
            raise Exception(res)


def alert_add_json(monitor, alert_to_create_json):
    ok, res = monitor.create_alert(alert_obj=alert_to_create_json)
    if not ok:
        raise Exception(res)

    return res['alert']



from sdccli.printer import print_list as _print_list, print_item as _print_item


def formats():
    return {
        "panelList": print_panel_list,
        "panel": print_panel,
    }


_list_keys = ["id", "name", "type", "queryCount"]
_item_keys = ["id", "name", "type", "queryCount"]


def print_panel(panel):
    queries_parent = panel["advancedQueries"] if "advancedQueries" in panel else panel["basicQueries"]
    panel["queryCount"] = len(queries_parent)
    _print_item(panel, _item_keys)

    print("queries:")
    queries = [{"id": id, "query": query.get("query") or ", ".join([q["id"] for q in query.get('metrics')])} for
               id, query in enumerate(queries_parent)]
    _print_list(queries, ["", "id", "query"])


def print_panel_list(panel_list):
    for panel in panel_list:
        if "queryCount" in panel:
            panel["queryCount"] = len(panel["advancedQueries"]) if "advancedQueries" in panel else len(
                panel["basicQueries"])

    _print_list(panel_list, _list_keys)



import click
import sys
from sdccli.cli.alert import alert
from sdccli.cli.backup import backup
from sdccli.cli.capture import capture
from sdccli.cli.command import command
from sdccli.cli.compliance import compliance
from sdccli.cli.dashboard import dashboard
from sdccli.cli.eventv1 import eventv1
from sdccli.cli.event import event
from sdccli.cli.policy import policy
from sdccli.cli.scanning import scanning
from sdccli.cli.settings import settings
from sdccli.cli.profile import profile
from sdccli.cli.dashboardv2 import dashboard_v2
from sdccli.config import Config
from sdccli import __version__


if sys.version_info.major < 3:
    print("Warning: You are using python {}.{}, it's recommended to use python 3".format(sys.version_info.major, sys.version_info.minor))
    print("         reinstall it using python3:")
    print("         $ python3 setup.py install")
    print("")


@click.group()
@click.option("-c", "--config",
              help="Uses the provided file as a config file. If the config file is not provided, it will be "
                   "searched at ~/.config/sdc-cli/config.yml and /etc/sdc-cli/config.yml.")
@click.option("-e", "--env", help="Uses a preconfigured environment in the config file. If it's not provided, "
                                  "it will use the 'main' environment or retrieve it from the env var SDC_ENV.")
@click.option("--json", is_flag=True, help="Output raw API JSON")
@click.version_option(__version__, '-v', '--version')
@click.pass_context
def cli(ctx, config, env, json):
    """
    You can provide the monitor/secure tokens by the SDC_MONITOR_TOKEN and SDC_SECURE_TOKEN environment variables.
    """
    cnf = Config(json)
    cnf.load(config, env)
    ctx.obj = cnf


cli.add_command(alert)
cli.add_command(backup)
cli.add_command(capture)
cli.add_command(command)
cli.add_command(compliance)
cli.add_command(dashboard)
cli.add_command(eventv1)
cli.add_command(event)
cli.add_command(policy)
cli.add_command(scanning)
cli.add_command(profile)
cli.add_command(settings)
cli.add_command(dashboard_v2)

if getattr(sys, 'frozen', False):
    cli(sys.argv[1:])



import click
import json
import sys
from sdccli.printer import print_list, print_item
from sdccli.usecases.scanning import alert as use_case

_item_keys = ["alertId", "name", "enabled", "description", "scope", "triggers",
              "notificationChannelNames", "createdAt", "updatedAt"]
_list_keys = ["alertId", "name", "enabled"]


@click.group(name='alert', short_help='Alert operations')
def alert():
    pass


@alert.command(name='add', short_help="Add an alert")
@click.argument('name', nargs=1)
@click.option('--description', nargs=1, help="The descprition of the alert.")
@click.option('--scope', nargs=1,
              help="An AND-composed string of predicates that selects the scope in which the alert will be "
                   """applied. (like: 'host.domain = "example.com" and container.image != "alpine:latest"')""")
@click.option('--nounscanned', is_flag=True, help="Disable unscanned image trigger'.")
@click.option('--nofailed', is_flag=True, help="Disable failed policy result trigger.")
@click.option('--noenable', is_flag=True, help="Whether this alert should actually be applied.")
@click.option('--notification', nargs=1, multiple=True, help="Notification channel id.")
@click.pass_obj
def add(cnf, name, description, scope, nounscanned, nofailed, noenable, notification):
    """
    NAME: The name of the alert.
    """
    try:
        res = use_case.add_scanning_alert(cnf.sdscanning, name,
                                          description=description,
                                          scope=scope,
                                          unscanned=not nounscanned,
                                          failed=not nofailed,
                                          enabled=not noenable,
                                          notification_channels=notification)
        cnf.formatter.format(res, "scanningAlert")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@alert.command(name='get', short_help="Get an alert")
@click.argument('alertid', nargs=1)
@click.pass_obj
def get(cnf, alertid):
    """
    ALERTID: Alert ID to get
    """
    try:
        res = use_case.get_alert_by_id(cnf.sdscanning, alertid)
        cnf.formatter.format(res, "scanningAlert")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@alert.command(name='list', short_help="List alerts")
@click.pass_obj
def list(cnf):
    ok, res = cnf.sdscanning.list_alerts()
    try:
        res = use_case.list_scanning_alerts(cnf.sdscanning)
        cnf.formatter.format(res, "scanningAlertList")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@alert.command(name='update', short_help="Update an alert")
@click.argument('alertid', nargs=1)
@click.argument('alertfile', type=click.Path(exists=True), nargs=1)
@click.pass_obj
def upd(cnf, alertid, alertfile):
    """
    ALERTID: Alert ID to get

    ALERTFILE: A file with a json description of the Alert
    """
    try:
        with open(alertfile) as f:
            alert = json.load(f)
    except Exception as error:
        print("Error parsing alert file (%s): %s" % (alertfile, str(error)))
        sys.exit(1)

    try:
        res = use_case.update_scanning_alert(cnf.sdscanning, alertid, alert)
        cnf.formatter.format(res, "scanningAlert")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@alert.command(name='del', short_help="Delete an alert")
@click.argument('alertid', nargs=1)
@click.pass_obj
def delete(cnf, alertid):
    """
    ALERTID: Alert ID to get
    """
    try:
        res = use_case.delete_scanning_alert(cnf.sdscanning, alertid)
    except Exception as ex:
        print(f"Error: {str(res)}")
        sys.exit(1)

    print("Success")



import click

import sdccli.usecases.scanning.vulnerability as use_case


@click.group(name='bundle', short_help='Vulnerability bundle operations')
def bundle():
    pass


@bundle.command(name='list', short_help="Lists all the vulnerability exception bundles")
@click.pass_obj
def list_bundles(cnf):
    """
    Lists all the vulnerability exception bundles
    """
    try:
        res = use_case.list_vulnerability_exception_bundles(cnf.sdscanning)
        cnf.formatter.format(res, "vulnExceptionBundlesList")
    except Exception as ex:
        print(ex)


@bundle.command(name='add', short_help="Add a vulnerability exception bundle")
@click.argument('name', nargs=1)
@click.option('--description', nargs=1, help="The descprition of vulnerability exception bundle.")
@click.pass_obj
def add(cnf, name, description=""):
    try:
        res = use_case.add_vulnerability_exception_bundle(cnf.sdscanning, name, description)
        cnf.formatter.format(res, "vulnExceptionBundlesAdded")
    except Exception as ex:
        print(ex)


@bundle.command(name='get', short_help="Retrieves information from a vulneability exception bundle")
@click.argument('bundleid', nargs=1)
@click.pass_obj
def get(cnf, bundleid):
    """
    BUNDLEID: Bundle ID to retrieve info from
    """
    try:
        res = use_case.get_vulnerability_exception_bundle(cnf.sdscanning, bundleid)
        cnf.formatter.format(res, "vulnExceptionBundle")
    except Exception as ex:
        print(ex)

@bundle.command(name="del", short_help="Removes a vulnerability exception bundle")
@click.argument('bundleid', nargs=1)
@click.pass_obj
def delete(cnf, bundleid):
    """
    BUNDLEID: Bundle ID to remove
    """
    try:
        use_case.delete_vulnerability_exception_bundle(cnf.sdscanning, bundleid)
        print("Succeed")
    except Exception as ex:
        print(ex)


import json
import sys

import click

import sdccli.helpers as helpers
import sdccli.usecases.dashboard.dashboard_v2 as use_case


@click.group(name='dashboard_v2', short_help='Sysdig Monitor dashboard operations')
def dashboard_v2():
    pass


@dashboard_v2.command(name='add', short_help="Add a dashboard")
@click.argument('name', nargs=1)
@click.argument('templatefile', nargs=1, type=click.Path(exists=True))
@click.option('--scope', nargs=1,
              help="filter to apply to the dashboard; must be based on metadata available in Sysdig Monitor; Example: *kubernetes.namespace.name='production' and container.image='nginx'*.")
@click.option('--shared', is_flag=True, help="if set the new dashboard will be a shared one.")
@click.option('--public', is_flag=True, help="if set the new dashboard will be shared with public token.")
@click.option('--annotation', nargs=1, multiple=True,
              help="a pair 'key=value' custom property that you can associate to this dashboard for automation or management reasons")
@click.pass_obj
def add(cnf, name, templatefile, scope, shared, public, annotation):
    """
    NAME: name for the new dashboard

    TEMPLATEFILE: A file with a json with "layout" and "items" elements
    """
    try:
        with open(templatefile) as f:
            template = json.load(f)
    except Exception as error:
        print("Error parsing rules file (%s): %s" % (templatefile, str(error)))
        sys.exit(1)

    try:
        annotations = helpers.annotation_arguments_to_map(annotation)
        res = use_case.add_dashboard(cnf.sdmonitor_v2, name, template, scope, shared, public, annotations)
        cnf.formatter.format(res, "dashboard")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@dashboard_v2.command(name='add-panel',
                      short_help="Adds a panel to the dashboard. A panel can be a time series, or a top chart (i.e. bar chart), or a number panel.")
@click.argument('dashboard', nargs=1)
@click.argument('name', nargs=1)
@click.option('--type', nargs=1, default="timeSeries",
              help="type of the new panel. Valid values are: 'timeSeries', 'top', 'number'")
@click.option('--metric', nargs=1, multiple=True, required=True,
              help="Specify a metric to show in the panel. A metric comes in the form of 'metricId:aggregation:groupKey'. A metricId is any of the entries that can "
                   "be found in the Metrics section of the Explore page in Sysdig Monitor. A grouping key is any of the entries that can be found in the Show or "
                   "Segment By sections of the Explore page in Sysdig Monitor. Note, certain panels allow certain combinations: timeSeries (1+ metrics or 1 metric + "
                   "grouping), top (1+ metrics or 1 metric + 1 grouping), number (1 metric only)")
@click.option('--scope', nargs=1,
              help="filter to apply to the dashboard; must be based on metadata available in Sysdig Monitor; Example: *kubernetes.namespace.name='production' and container.image='nginx'*.")
@click.option('--sort', nargs=1,
              help="Data sorting; The parameter is optional and it's a dictionary of ``metric`` and ``mode`` (it can be ``desc`` or ``asc``)")
@click.option('--limit', nargs=1, type=click.INT,
              help="This parameter sets the limit on the number of lines/bars shown in a 'timeSeries' or 'top' panel. In the case of more entities being available "
                   "than the limit, the top entities according to the sort will be shown. The default value is 10 for 'top' panels (for 'timeSeries' the default is "
                   "defined by Sysdig Monitor itself). Note that increasing the limit above 10 is not officially supported and may cause performance and rendering issues")
@click.option('--layout', nargs=1,
              help="Size and position of the panel. The dashboard layout is defined by a grid of 12 columns, each row height is equal to the column height. For example, "
                   "say you want to show 2 panels at the top: one panel might be 6 x 3 (half the width, 3 rows height) located in row 1 and column 1 (top-left corner of "
                   "the viewport), the second panel might be 6 x 3 located in row 1 and position 7. The location is specified by 'row:col:width:height'")
@click.pass_obj
def add_panel(cnf, dashboard, name, type, metric, scope, sort, limit, layout):
    """
    DASHBOARD: dashboard (id or name) to edit

    NAME: name of the new panel
    """
    try:
        dashboard = use_case.get_dashboard_by_id_or_name(cnf.sdmonitor_v2, dashboard).raw()

        panel_builder = use_case.PanelBuilder()
        if layout:
            parts = layout.split(':')
            if len(parts) != 4:
                raise Exception("layout must be in the form of 'row:col:width:height'")
            panel_builder.with_layout(*parts)

        if metric:
            for m in metric:
                parts = m.split(':')
                panel_builder.with_metric(*parts)

        panel = panel_builder.with_name(name).with_type(type).with_scope(scope).with_limit(limit).with_sort(
            sort).build()

        dashboard = use_case.add_panel_to_dashboard(cnf.sdmonitor_v2, dashboard, panel)
        cnf.formatter.format(dashboard, "dashboard")

    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@dashboard_v2.command(name='add-json', short_help="Add dashboards from a json file")
@click.argument('jsonfile', nargs=1, type=click.Path(exists=True))
@click.pass_obj
def add_json(cnf, jsonfile):
    """
    JSONFILE: A file with a json description of one or multiple dashboards
    """
    try:
        with open(jsonfile) as f:
            dashboard_json = json.load(f)
    except Exception as error:
        print("Error parsing rules file (%s): %s" % (jsonfile, str(error)))
        sys.exit(1)

    try:
        res = use_case.add_json_dashboard(cnf.sdmonitor_v2, dashboard_json)
        cnf.formatter.format(res, "dashboard")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@dashboard_v2.command(name='list', short_help="List all dashboards")
@click.pass_obj
def list_dashboards(cnf):
    try:
        res = use_case.list_dashboards(cnf.sdmonitor_v2)
        cnf.formatter.format(res, "dashboardList")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@dashboard_v2.command(name='get', short_help="Get dashboard")
@click.argument('dashboard', nargs=1)
@click.pass_obj
def get(cnf, dashboard):
    """
    DASHBOARD: The dashboard (id or name) to get
    """
    try:
        res = use_case.get_dashboard_by_id_or_name(cnf.sdmonitor_v2, dashboard)
        cnf.formatter.format(res, "dashboard")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@dashboard_v2.command(name='del', short_help="Delete dashboards")
@click.argument('dashboards', nargs=-1, required=True)
@click.pass_obj
def delete(cnf, dashboards):
    """
    DASHBOARDS: Dashboard id to delete
    """
    try:
        use_case.delete_dashboards_by_id(cnf.sdmonitor_v2, dashboards)
        print("Success")
    except Exception as ex:
        print(f"Error deleting dashboard: {str(ex)}")


@dashboard_v2.command(name='del-panel', short_help="Removes a panel from the dashboard")
@click.argument('dashboard', nargs=1)
@click.argument('name', nargs=1)
@click.pass_obj
def delete_panel(cnf, dashboard, name):
    """
    DASHBOARD: dashboard (id or name) to delete the panel from

    NAME: name of the panel to find and remove
    """
    try:
        dboard = use_case.get_dashboard_by_id_or_name(cnf.sdmonitor_v2, dashboard)
        use_case.remove_panel_from_dashboard(cnf.sdmonitor_v2, dboard, name)
        print("Success")
    except Exception as ex:
        print(f"Error {str(ex)}")
        sys.exit(1)



import click
import json
import sys

from sdccli.usecases.alert import alert_add, alert_delete, alert_get, alert_update, alert_list, alert_add_json


@click.group(name='alert', short_help='Sysdig Monitor alert operations')
def alert():
    pass


@alert.command(name='add', short_help="Add an alert")
@click.argument('name', nargs=1)
@click.option('--description', nargs=1, default="",
              help="The alert description. This will appear in the Sysdig Monitor UI and in notification emails.")
@click.option('--severity', nargs=1, type=click.INT, default=4,
              help="syslog-encoded alert severity. This is a number from 0 to 7 where 0 means 'emergency' and 7 is 'debug'. (by default is 4)")
@click.option('--atleast', nargs=1, type=click.INT, default=600,
              help="the number of consecutive seconds the condition must be satisfied for the alert to fire. (by default is 600)")
@click.option('--condition', nargs=1, default="",
              help="the alert condition, as described here https://app.sysdigcloud.com/apidocs/#!/Alerts/post_api_alerts")
@click.option('--disable', is_flag=True, help="The alert will be disabled when created. (by default is enabled)")
@click.option('--segment', nargs=1, multiple=True,
              help="A segmentation criteria that can be used to apply the alert to multiple entities.")
@click.option('--segment-condition', nargs=1, default="ANY",
              help="When segment is specified (and therefore the alert will cover multiple entities) this field is used to determine when it will fire. In particular, you have two options for segment-condition: 'ANY' (the alert will fire when at least one of the monitored entities satisfies the condition) and 'ALL' (the alert will fire when all of the monitored entities satisfy the condition).")
@click.option('--user-filter', nargs=1, default="",
              help="a boolean expression combining Sysdig Monitor segmentation criteria that makes it possible to reduce the scope of the alert. For example: kubernetes.namespace.name='production' and container.image='nginx'")
@click.option('--notify', nargs=1,
              help="the type of notification you want this alert to generate. Options are 'EMAIL', 'SNS', 'PAGER_DUTY', 'SYSDIG_DUMP'")
@click.option('--annotation', nargs=1, multiple=True,
              help="a pair 'key=value' custom property that you can associate to this alert for automation or management reasons")
@click.option('--promql', is_flag=True,
              help="Define if the alert to be created follows the PromQL syntax")
@click.pass_obj
def add(cnf, name, description, severity, atleast, condition, disable, segment, segment_condition,
        user_filter, notify, annotation, promql):
    try:
        type = "PROMETHEUS" if promql else "MANUAL"
        res = alert_add(cnf.sdmonitor, name, description, severity, atleast, condition, disable, segment,
                        segment_condition, user_filter, notify, annotation, type)
        cnf.formatter.format(res, "alert")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@alert.command(name='add-json', short_help="Add an alert from a json file")
@click.argument('alertfile', type=click.Path(exists=True), nargs=1)
@click.pass_obj
def add_json(cnf, alertfile):
    """
    ALERTFILE: A file with a json alert description.
    """
    try:
        with open(alertfile) as f:
            alert = f.read()
    except Exception as error:
        print("Error parsing alert (%s): %s" % (alertfile, str(error)))
        sys.exit(1)

    try:
        res = alert_add_json(cnf.sdmonitor, json.loads(alert))
        cnf.formatter.format(res, "alert")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@alert.command(name='list', short_help="List all alerts")
@click.pass_obj
def list(cnf):
    try:
        res = alert_list(cnf.sdmonitor)
        cnf.formatter.format(res, "alertList")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@alert.command(name='update', short_help="Update an alert")
@click.argument('alertfile', type=click.Path(exists=True), nargs=1)
@click.pass_obj
def upd(cnf, alertfile):
    """
    ALERTFILE: A file with a json alert description.
    """
    try:
        with open(alertfile) as f:
            alert = json.load(f)
    except Exception as error:
        print("Error parsing rules file (%s): %s" % (alertfile, str(error)))
        sys.exit(1)

    try:
        res = alert_update(cnf.sdmonitor, alert)
        cnf.formatter.format(res, "alert")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@alert.command(name='get', short_help="Get alert")
@click.argument('alert', nargs=1)
@click.pass_obj
def get(cnf, alert):
    try:
        res = alert_get(cnf.sdmonitor, alert)
        cnf.formatter.format(res, "alert")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@alert.command(name='del', short_help="Delete alerts")
@click.argument('alerts', nargs=-1, type=click.INT, required=True)
@click.pass_obj
def delete(cnf, alerts):
    try:
        alert_delete(cnf.sdmonitor, alerts)
        print("Success")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)



import click
import json
import sys
from sdccli.printer import print_list, print_item


_item_keys = ["id", "username", "firstName", "lastName", "enabled", "systemRole", "products"]
_list_keys = ["id", "username", "firstName", "lastName", "enabled"]


@click.group(name='user', short_help='Sysdig user operations')
def user():
    pass


@user.command(name='add', short_help="Invites a new user. This should result in an email notification to the specified address.")
@click.argument('email', nargs=1)
@click.argument('first_name', nargs=1)
@click.argument('last_name', nargs=1)
@click.option('--system-role', nargs=1, default="ROLE_USER",
              help="System-wide privilege level for this user regardless of team. specify 'ROLE_CUSTOMER' "
                   "to create an Admin. If not specified, default is a non-Admin ('ROLE_USER').")
@click.pass_obj
def add(cnf, email, first_name, last_name, system_role):
    """
    EMAIL: The email address of the user that will be invited to use Sysdig Monitor
    FIRST_NAME: The first name of the user being invited
    LAST_NAME: The last name of the user being invited
    """
    ok, res = cnf.sd.create_user_invite(email, first_name=first_name, last_name=last_name, system_role=system_role)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res["user"], _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@user.command(name='list', short_help="List all users")
@click.pass_obj
def list(cnf):
    ok, res = cnf.sd.get_users()

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_list(res, _list_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@user.command(name='update', short_help="Update the user information")
@click.argument('email', nargs=1)
@click.option('--first-name', nargs=1, help="The first name to be set")
@click.option('--last-name', nargs=1, help="The last name to be set")
@click.option('--system-role', nargs=1, default="ROLE_USER",
              help="System-wide privilege level for this user regardless of team. specify 'ROLE_CUSTOMER' "
                   "to create an Admin.")
@click.pass_obj
def upd(cnf, email, first_name, last_name, system_role):
    """
    EMAIL: The email address of the user that will be updated.
    """
    ok, res = cnf.sd.edit_user(email, firstName=first_name, lastName=last_name, systemRole=system_role)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print("Success")
    else:
        print("Error: " + str(res))
        sys.exit(1)


@user.command(name='me', short_help="Get my user info")
@click.pass_obj
def me(cnf):
    ok, res = cnf.sd.get_user_info()

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res["user"], _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@user.command(name='get', short_help="Get user")
@click.argument('email', nargs=1)
@click.pass_obj
def get(cnf, email):
    """
    EMAIL: email of the user to get
    """
    ok, res = cnf.sd.get_user(email)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res, _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@user.command(name='del', short_help="Delete users")
@click.argument('emails', nargs=-1, required=True)
@click.pass_obj
def delete(cnf, emails):
    """
    EMAILS: The emails of the users to delete
    """
    for email in emails:
        ok, res = cnf.sd.delete_user(email)

        if not ok:
            print("Error: " + str(res))
            sys.exit(1)

    print("Success")



import click
import json
import sys
from sdccli.printer import print_list, print_item


_item_keys = ["id", "name", "condition", "filename"]
_list_keys = ["ids", "name"]


@click.group(name='falco-macro', short_help='Sysdig Secure policy falco macro operations')
def falco_macro():
    pass


@falco_macro.command(name='add', short_help="Add a falco macro")
@click.argument('name', nargs=1)
@click.argument('condition', nargs=1)
@click.pass_obj
def add(cnf, name, condition):
    """
    NAME: the name of the new falco macro.

    CONDITION: the full condition text exactly as represented in the yaml file.
    """
    if not cnf.sdsecure.policy_v2:
        print("Error: Sysdig Secure doesn't support policy API v2")
        sys.exit(1)

    ok, res = cnf.sdsecure.add_falco_macro(name, condition)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res, _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@falco_macro.command(name='list', short_help="List all falco macros")
@click.pass_obj
def lst(cnf):
    if not cnf.sdsecure.policy_v2:
        print("Error: Sysdig Secure doesn't support policy API v2")
        sys.exit(1)

    ok, res = cnf.sdsecure.list_falco_macros()

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_list(res, _list_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@falco_macro.command(name='get', short_help="Get falco macro")
@click.argument('falco_macro', nargs=1)
@click.pass_obj
def get(cnf, falco_macro):
    """
    FALCO_MACRO: Falco macro (id or name) to get
    """
    if not cnf.sdsecure.policy_v2:
        print("Error: Sysdig Secure doesn't support policy API v2")
        sys.exit(1)

    try:
        id = int(falco_macro)
        ok, res = cnf.sdsecure.get_falco_macro_id(id)
    except ValueError:
        ok, res = cnf.sdsecure.get_falco_macros_group(falco_macro)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        if isinstance(res, list):
            for rule in res:
                print_item(rule, _item_keys)
        else:
            print_item(res, _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@falco_macro.command(name='update', short_help="Update falco list")
@click.argument('falco_macro', nargs=1)
@click.argument('condition', nargs=1)
@click.pass_obj
def upd(cnf, falco_macro, condition):
    """
    FALCO_LIST: Falco list id to update

    CONDITION: the full condition text exactly as represented in the yaml file.
    """
    if not cnf.sdsecure.policy_v2:
        print("Error: Sysdig Secure doesn't support policy API v2")
        sys.exit(1)

    ok, res = cnf.sdsecure.update_falco_macro(falco_macro, condition)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res, _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@falco_macro.command(name='del', short_help="Delete falco macro")
@click.argument('falco_macros', nargs=-1, required=True)
@click.pass_obj
def delete(cnf, falco_macros):
    """
    FALCO_MACROS: Falco macro ids to delete
    """
    if not cnf.sdsecure.policy_v2:
        print("Error: Sysdig Secure doesn't support policy API v2")
        sys.exit(1)

    for fm in falco_macros:
        ok, res = cnf.sdsecure.delete_falco_macro(fm)

        if not ok:
            print("Error: " + str(res))
            sys.exit(1)

    print("Success")



import click
import json
import sys
from sdccli.printer import print_list, print_item


_item_keys = ['subscription_id', 'subscription_key', "active", 'created_at', 'last_updated']
_list_keys = ["subscription_key", "active"]


@click.group(name='repo', short_help='Repository operations')
def repo():
    pass


@repo.command(name='add', short_help="Add a repository")
@click.option('--noautosubscribe', is_flag=True, help="If set, instruct the engine to disable subscriptions for any discovered tags.")
@click.option('--lookuptag', help="Specify a tag to use for repo tag scan if 'latest' tag does not exist in the repo.")
@click.argument('repo', nargs=1)
@click.pass_obj
def add(cnf, repo, noautosubscribe, lookuptag):
    """
    REPO: Input repository can be in the following formats: registry/repo
    """
    ok, res = cnf.sdscanning.add_repo(
        repo,
        autosubscribe=not noautosubscribe,
        lookuptag=lookuptag)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res[0], _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@repo.command(name='list', short_help="List added repositories")
@click.pass_obj
def listrepos(cnf):
    ok, res = cnf.sdscanning.list_repos()

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_list(res, _list_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@repo.command(name='get', short_help="Get a repository")
@click.argument('repo', nargs=1)
@click.pass_obj
def get(cnf, repo):
    """
    REPO: Input repository can be in the following formats: registry/repo
    """
    ok, res = cnf.sdscanning.get_repo(repo)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res[0], _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@repo.command(name='del', short_help="Delete a repository from the watch list (does not delete already analyzed images)")
@click.argument('repos', nargs=-1, required=True)
@click.pass_obj
def delete(cnf, repos):
    """
    REPOS: Input repo can be in the following formats: registry/repo
    """
    for repo in repos:
        ok, res = cnf.sdscanning.delete_repo(repo)

        if not ok:
            print("Error: " + str(res))
            sys.exit(1)

    print("Success")


@repo.command(name='unwatch', short_help="Instruct engine to stop automatically watching the repo for image updates")
@click.argument('repo', nargs=1)
@click.pass_obj
def unwatch(cnf, repo):
    """
    REPO: Input repo can be in the following formats: registry/repo
    """
    ok, res = cnf.sdscanning.unwatch_repo(repo)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res[0], _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@repo.command(name='watch', short_help="Instruct engine to start automatically watching the repo for image updates")
@click.argument('repo', nargs=1)
@click.pass_obj
def watch(cnf, repo):
    """
    REPO: Input repo can be in the following formats: registry/repo
    """
    ok, res = cnf.sdscanning.watch_repo(repo)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res[0], _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)



from sdcclient import SdMonitorClient

PANEL_VISUALIZATION_TIMECHART = "advancedTimechart"
PANEL_VISUALIZATION_NUMBER = "advancedNumber"


# TODO(fede): Move this to the Python SDK
def get_panels_from_dashboard(monitor: SdMonitorClient, dashboard_id):
    ok, res = monitor.get_dashboard(dashboard_id)
    if not ok:
        raise Exception(res)

    return res["dashboard"]["panels"]


# TODO(fede): Move this to the Python SDK
def delete_panel_from_dashboard(monitor: SdMonitorClient, dashboard_id, panel_id):
    ok, res = monitor.get_dashboard(dashboard_id)
    if not ok:
        raise Exception(res)

    res["dashboard"]["panels"] = [panel for panel in res["dashboard"]["panels"] if panel["id"] != panel_id]
    res["dashboard"]["layout"] = [layout for layout in res["dashboard"]["layout"] if layout["panelId"] != panel_id]

    ok, res = monitor.update_dashboard(res["dashboard"])
    if not ok:
        raise Exception(res)

# TODO(fede): Move this to the Python SDK
def create_panel_in_dashboard(monitor: SdMonitorClient, dashboard_id, panel_name, visualization, query):
    ok, res = monitor.get_dashboard(dashboard_id)
    if not ok:
        raise Exception(res)

    new_panel_id = res["dashboard"]["panels"][-1]["id"] + 1
    new_panel = {
        "id": new_panel_id,
        "type": visualization,
        "name": panel_name,
        "description": "",
        "advancedQueries": [
            {
                "enabled": True,
                "displayInfo": {
                    "displayName": "",
                    "timeSeriesDisplayNameTemplate": "",
                    "type": "lines"
                },
                "format": {
                    "unit": "%",
                    "inputFormat": "0-100",
                    "displayFormat": "auto",
                    "decimals": None,
                    "yAxis": "auto"
                },
                "query": query
            }
        ]
    }
    new_layout = {
        "panelId": new_panel_id,
        "x": 0,
        # Hackish way to position a panel, the API doesn't provide auto-position
        "y": len(res["dashboard"]["panels"]) * 12 + 12,
        "w": 12,
        "h": 6,
    }

    if visualization == PANEL_VISUALIZATION_TIMECHART:
        new_panel["axesConfiguration"] = {
            "bottom": {
                "enabled": True
            },
            "left": {
                "enabled": True,
                "displayName": None,
                "unit": "auto",
                "displayFormat": "auto",
                "decimals": None,
                "minValue": 0,
                "maxValue": None,
                "minInputFormat": "0-100",
                "maxInputFormat": "0-100",
                "scale": "linear"
            },
            "right": {
                "enabled": True,
                "displayName": None,
                "unit": "auto",
                "displayFormat": "auto",
                "decimals": None,
                "minValue": 0,
                "maxValue": None,
                "minInputFormat": "1",
                "maxInputFormat": "1",
                "scale": "linear"
            }
        }
        new_panel["legendConfiguration"] = {
            "enabled": True,
            "position": "right",
            "layout": "table",
            "showCurrent": True
        }
    if visualization == PANEL_VISUALIZATION_NUMBER:
        new_panel["numberThresholds"] = {
            "values": [],
            "base": {
                "severity": "none",
                "displayText": "",
            }
        }

    res["dashboard"]["panels"].append(new_panel)
    res["dashboard"]["layout"].append(new_layout)

    ok, res = monitor.update_dashboard(res["dashboard"])
    if not ok:
        raise Exception(res)

    return res["dashboard"]["panels"][-1]


# TODO(fede): Move this to the Python SDK
def get_queries_from_panel(monitor: SdMonitorClient, dashboard_id, panel_id):
    ok, res = monitor.get_dashboard(dashboard_id)
    if not ok:
        raise Exception(res)

    panel_list = [panel for panel in res["dashboard"]["panels"] if panel["id"] == panel_id]
    if len(panel_list) == 0:
        raise Exception(f"panel with id {panel_id} not found in dashboard {dashboard_id}")

    panel = panel_list[0]
    return [query["query"] for query in panel["advancedQueries"]]


# TODO(fede): Move this to the Python SDK
def add_query_to_panel(monitor: SdMonitorClient, dashboard_id, panel_id, query):
    ok, res = monitor.get_dashboard(dashboard_id)
    if not ok:
        raise Exception(res)

    panel_list = [panel for panel in res["dashboard"]["panels"] if panel["id"] == panel_id]
    if len(panel_list) == 0:
        raise Exception(f"panel with id {panel_id} not found in dashboard {dashboard_id}")

    panel = panel_list[0]
    if panel["type"] not in [PANEL_VISUALIZATION_TIMECHART]:
        raise Exception("cannot add query to this kind of panel")


    new_query = {
        "enabled": True,
        "displayInfo": {
            "displayName": "",
            "timeSeriesDisplayNameTemplate": "",
            "type": "lines"
        },
        "format": {
            "unit": "%",
            "inputFormat": "0-100",
            "displayFormat": "auto",
            "decimals": None,
            "yAxis": "auto"
        },
        "query": query
    }
    panel["advancedQueries"].append(new_query)

    ok, res = monitor.update_dashboard(res["dashboard"])
    if not ok:
        raise Exception(res)

    panel_list = [panel for panel in res["dashboard"]["panels"] if panel["id"] == panel_id]
    panel = panel_list[0]
    return [query["query"] for query in panel["advancedQueries"]]


# TODO(fede): Move this to the Python SDK
def del_query_from_panel(monitor: SdMonitorClient, dashboard_id, panel_id, query_id):
    ok, res = monitor.get_dashboard(dashboard_id)
    if not ok:
        raise Exception(res)

    panel_list = [panel for panel in res["dashboard"]["panels"] if panel["id"] == panel_id]
    if len(panel_list) == 0:
        raise Exception(f"panel with id {panel_id} not found in dashboard {dashboard_id}")

    panel = panel_list[0]

    if panel["type"] not in [PANEL_VISUALIZATION_TIMECHART, PANEL_VISUALIZATION_NUMBER]:
        raise Exception("removing queries from non-promql panels is not supported yet")

    if len(panel["advancedQueries"]) == 1:
        raise Exception(f"cannot remove the only query for a panel")

    if len(panel["advancedQueries"]) > (query_id + 1):
        raise Exception(f"query {query_id} not found in panel {panel_id} from dashboard {dashboard_id}")

    del panel["advancedQueries"][query_id]

    ok, res = monitor.update_dashboard(res["dashboard"])
    if not ok:
        raise Exception(res)

    panel_list = [panel for panel in res["dashboard"]["panels"] if panel["id"] == panel_id]
    panel = panel_list[0]
    return [query["query"] for query in panel["advancedQueries"]]




import click

from sdccli.cli.scanning.vulnerability import vulnerability
from sdccli.cli.scanning.image import image
from sdccli.cli.scanning.registry import registry
from sdccli.cli.scanning.repo import repo
from sdccli.cli.scanning.subscription import subscription
from sdccli.cli.scanning.policy import policy
from sdccli.cli.scanning.alert import alert
from sdccli.cli.scanning.runtime import runtime
from sdccli.cli.scanning.query import query
from sdccli.cli.scanning.cve_report import cve_report


@click.group(short_help='Scanning operations')
def scanning():
    pass


scanning.add_command(image)
scanning.add_command(registry)
scanning.add_command(repo)
scanning.add_command(subscription)
scanning.add_command(policy)
scanning.add_command(alert)
scanning.add_command(runtime)
scanning.add_command(query)
scanning.add_command(cve_report)
scanning.add_command(vulnerability)


import click
import json
import sys
from sdccli.printer import print_list, print_item


_item_keys = ["id", "type", "name", "enabled", "options"]
_list_keys = ["id", "name", "enabled"]


@click.group(name='notification', short_help='Sysdig notification channel operations')
def notification():
    pass


@notification.command(name='add', short_help="Create a new notification channel")
@click.argument('name', nargs=1)
@click.option('--email', nargs=1, multiple=True, required=True, help="Email recipients")
@click.pass_obj
def add(cnf, name, email):
    """
    NAME: the name of the notification channel to create.
    """
    ok, res = cnf.sd.create_email_notification_channel(name, email)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res["notificationChannel"], _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@notification.command(name='list', short_help="List all notification channels")
@click.pass_obj
def list(cnf):
    ok, res = cnf.sd.get_notification_ids()
    if not ok:
        print("Error: " + str(res))
        sys.exit(1)

    channels = []
    for id in res:
        ok, res = cnf.sd.get_notification_channel(id)
        if not ok:
            print("Error: " + str(res))
            sys.exit(1)
        channels.append(res)

    if cnf.json:
        print(json.dumps(channels, indent=4))
        return

    print_list(channels, _list_keys)


@notification.command(name='update', short_help="Update notification channel")
@click.argument('jsonfile', nargs=1, type=click.Path(exists=True))
@click.pass_obj
def upd(cnf, jsonfile):
    """
    JSONFILE: A file with a json description of the notification channel to update
    """
    try:
        with open(jsonfile) as f:
            notification = json.load(f)
    except Exception as error:
        print("Error parsing rules file (%s): %s" % (jsonfile, str(error)))
        sys.exit(1)

    ok, res = cnf.sd.update_notification_channel(notification)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res["notificationChannel"], _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@notification.command(name='get', short_help="Get a notification channel")
@click.argument('id', nargs=1)
@click.pass_obj
def get(cnf, id):
    """
    ID: the id to get
    """
    ok, res = cnf.sd.get_notification_channel(id)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res, _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@notification.command(name='del', short_help="Delete notification channels")
@click.argument('ids', nargs=-1, required=True)
@click.pass_obj
def delete(cnf, ids):
    """
    IDS: The ids of the notification channels to delete
    """
    for id in ids:
        ok, res = cnf.sd.delete_notification_channel({"id": id})

        if not ok:
            print("Error: " + str(res))
            sys.exit(1)

    print("Success")



from sdcclient import SdMonitorClient


def retrieve_team_by_name_or_id(monitor: SdMonitorClient, team):
    try:
        # By ID
        id = int(team)
        ok, res = monitor.get_teams()
        if not ok:
            raise Exception(res)

        for d in res:
            if id == d["id"]:
                return d

        raise Exception(f"no team with id {id} found")
    except ValueError:
        # By Name
        ok, res = monitor.get_team(team)
        if not ok:
            raise Exception(res)

        return res


from sdcclient import SdScanningClient


def add_scanning_alert(scanning, name, description, scope, unscanned, failed, enabled, notification_channels):
    ok, res = scanning.add_alert(
        name=name,
        description=description,
        scope=scope,
        triggers={'unscanned': unscanned, 'failed': failed},
        enabled=enabled,
        notification_channels=notification_channels)
    if not ok:
        raise Exception(res)

    return res


def get_alert_by_id(scanning, alert_id):
    ok, res = scanning.get_alert(alert_id)
    if not ok:
        raise Exception(res)
    return res


def list_scanning_alerts(scanning: SdScanningClient):
    ok, res = scanning.list_alerts()
    if not ok:
        return Exception(res)

    return res["alerts"]


def update_scanning_alert(scanning: SdScanningClient, alert_id, alert_data):
    ok, res = scanning.update_alert(alert_id, alert_data)
    if not ok:
        raise Exception(res)
    return res;


def delete_scanning_alert(scanning: SdScanningClient, alert_id):
    ok, res = scanning.delete_alert(alert_id)
    if not ok:
        raise Exception(res)
    return res;



import click
import json
import sys
from sdccli.printer import print_list, print_item


_item_keys = ["id", "name", "comment", "rules"]
_list_keys = ["id", "name", "comment"]


@click.group(name='policy', short_help='Policy operations')
def policy():
    pass


@policy.command(name='add', short_help="Add a policy")
@click.argument('name', nargs=1)
@click.argument('rulesfile', type=click.Path(exists=True), nargs=1)
@click.option('--comment', nargs=1, help="A human-readable description.")
@click.option('--bundleid', nargs=1, help="Target bundle. If not specified, the currently active bundle will be used.")
@click.pass_obj
def add(cnf, name, rulesfile, comment, bundleid):
    """
    NAME: name for the new policy

    RULESFILE: A file with a json list of PolicyRule elements (while creating/updating a policy, new rule IDs will be created backend side)
    """
    # TODO: optionally pass the rules as string?
    try:
        with open(rulesfile) as f:
            rules = json.load(f)
    except Exception as error:
        print("Error parsing rules file (%s): %s" % (rulesfile, str(error)))
        sys.exit(1)

    ok, res = cnf.sdscanning.add_policy(name, rules, comment=comment, bundleid=bundleid)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res, _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@policy.command(name='list', short_help="List all policies")
@click.option('--bundleid', nargs=1, help="Target bundle. If not specified, the currently active bundle will be used.")
@click.pass_obj
def list(cnf, bundleid):
    ok, res = cnf.sdscanning.list_policies(bundleid=bundleid)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_list(res, _list_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@policy.command(name='get', short_help="Get a policy")
@click.argument('policyid', nargs=1)
@click.option('--bundleid', nargs=1, help="Target bundle. If not specified, the currently active bundle will be used.")
@click.pass_obj
def get(cnf, policyid, bundleid):
    """
    POLICYID: Policy ID to get
    """
    ok, res = cnf.sdscanning.get_policy(policyid, bundleid)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res, _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@policy.command(name='update', short_help="Update a policy")
@click.argument('policyid', nargs=1)
@click.argument('policyfile', type=click.Path(exists=True), nargs=1)
@click.option('--bundleid', nargs=1, help="Target bundle. If not specified, the currently active bundle will be used.")
@click.pass_obj
def upd(cnf, policyid, policyfile, bundleid):
    """
    POLICYID: Policy ID to update

    POLICYFILE: A file with a json description of the Policy
    """
    try:
        with open(policyfile) as f:
            policy = json.load(f)
    except Exception as error:
        print("Error parsing policy file (%s): %s" % (policyfile, str(error)))
        sys.exit(1)

    ok, res = cnf.sdscanning.update_policy(policyid, policy)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res, _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@policy.command(name='del', short_help="Delete a policy")
@click.argument('policyid', nargs=1)
@click.option('--bundleid', nargs=1, help="Target bundle. If not specified, the currently active bundle will be used.")
@click.pass_obj
def delete(cnf, policyid, bundleid):
    """
    POLICYID: Policy ID to delete
    """
    ok, res = cnf.sdscanning.delete_policy(policyid, bundleid=bundleid)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print("Success")
    else:
        print("Error: " + str(res))
        sys.exit(1)



from datetime import datetime

from dateutil import parser

from sdccli.printer import print_list, print_item

_severity_mapping = ["HIGH", "HIGH", "HIGH", "HIGH", "MED", "MED", "LOW", "INFO"]


def formats():
    return {
        "policyEventV1List": print_policy_events_v1,
        "policyEventV1": print_policy_event_v1
    }


def print_policy_event_v1(event):
    event["date"] = datetime.strftime(parser.parse(event["timestamp"]), "%Y-%m-%d %H:%M:%S %Z")
    event["type"] = event["originator"]
    event["severity"] = _severity_mapping[event["severity"]]
    ruleTags = event["content"].get("ruleTags", []) or []
    event["tags"] = ", ".join(ruleTags)
    event["output"] = event["content"].get("output", "")
    print_item(event, ["id", "name", "description", "date", "type", "severity", "output", "tags"])

    if "fields" in event["content"]:
        print("fields:")
        max_size = max(len(k) for k in event["content"]["fields"].keys())
        for key, value in event["content"]["fields"].items():
            print(f"  {key:{max_size}}  {value}")

    if "image" in event["content"]:
        print("image:")

        max_size = max(len(k) for k in event["content"]["image"].keys())
        for key, value in event["content"]["image"].items():
            print(f"  {key:{max_size}}  {value}")



    if "labels" in event:
        print("labels:")
        max_size = max(len(k) for k in event["labels"].keys())
        for key, value in event["labels"].items():
            print(f"  {key:{max_size}}  {value}")


def print_policy_events_v1(data):
    if not data:
        print("No policy events found")

    policy_events = [{
        "id": event["id"],
        "name": event["name"],
        "severity": _severity_mapping[event["severity"]],
        "type": event["originator"],
        "date": datetime.strftime(parser.parse(event["timestamp"]), "%Y-%m-%d %H:%M:%S %Z"),
    } for event in data]

    print_list(policy_events, ["id", "name", "severity", "type", "date"])



import os
import os.path
import sys
from pathlib import Path

import click

from sdccli.usecases.backup import *


@click.group(name='backup', short_help="Backup operations")
def backup():
    pass


@backup.command(name='dump', short_help="Dumps all the information from Monitor and Secure to a directory.")
@click.option('-f', 'overwrite', is_flag=True, help="Overwrite file if it already exists")
@click.argument('backup-file', nargs=1)
@click.argument('section', nargs=-1)
@click.pass_obj
def dump(cnf, overwrite, backup_file, section):
    """
    BACKUP_FILE: File where the data must be saved to.

    SECTION: Restore only the specified section. Valid values: dashboards, notification_channels, alerts, users, teams_monitor, teams_secure, policies, falco_rules, falco_lists, falco_macros
    """
    backup_file_path = Path(backup_file)
    if backup_file_path.exists() and not overwrite:
        print("Error: file {} already exists. Use -f if you want to overwrite it.".format(backup_file))
        sys.exit(1)

    if not backup_file_path.parent.exists():
        backup_file_path.parent.mkdir(parents=True)

    if backup_file_path.exists() and not backup_file_path.is_file():
        print("Error: {} is not file".format(backup_file))
        sys.exit(1)

    repository = open(backup_file, mode='w')
    use_cases_to_execute = []

    if not section or 'dashboards' in section:
        use_cases_to_execute.append(DumpDashboardsUseCase(cnf.sdmonitor, repository))

    if not section or 'notification_channels' in section:
        use_cases_to_execute.append(DumpNotificationChannelsUseCase(cnf.sdmonitor, repository))

    if not section or 'alerts' in section:
        use_cases_to_execute.append(DumpAlertsUseCase(cnf.sdmonitor, repository))

    if not section or 'users' in section:
        use_cases_to_execute.append(DumpUsersUseCase(cnf.sdmonitor, repository))

    if not section or 'teams_monitor' in section:
        use_cases_to_execute.append(DumpTeamsMonitorUseCase(cnf.sdmonitor, repository))

    if not section or 'teams_secure' in section:
        use_cases_to_execute.append(DumpTeamsSecureUseCase(cnf.sdsecure, repository))

    if not section or 'policies' in section:
        if cnf.sdsecure.policy_v2:
            use_cases_to_execute.append(DumpPoliciesUseCase(cnf.sdsecure, repository))
        else:
            use_cases_to_execute.append(DumpPoliciesUseCase(cnf.sdsecure_v1, repository))

    if not section or 'falco_rules' in section:
        use_cases_to_execute.append(DumpUserCreatedRulesUseCase(cnf.sdsecure, repository))

    if not section or 'falco_lists' in section:
        use_cases_to_execute.append(DumpUserCreatedFalcoListsUseCase(cnf.sdsecure, repository))

    if not section or 'falco_macros' in section:
        use_cases_to_execute.append(DumpUserCreatedFalcoMacrosUseCase(cnf.sdsecure, repository))

    errors = []
    for use_case in use_cases_to_execute:
        ok, res = use_case.execute()
        if not ok:
            errors.append(res)

    if len(errors) > 0:
        for error in errors:
            print(error)
        sys.exit(1)
    sys.exit(0)


@backup.command(name='check', short_help="Checks if something has changed in the remote environment "
                                         "comparing it with the backed up version")
@click.argument('backup-path', nargs=1)
@click.pass_obj
def check(cnf, backup_path):
    """
    BACKUP_PATH: Directory where the data must be saved to.
    """
    if not os.path.isdir(backup_path):
        print("Error: {} is not a correct directory".format(backup_path))
        sys.exit(1)

    for name, func in _getters(cnf).items():
        filename = os.path.join(backup_path, name + ".json")
        with open(filename) as file:
            ok, remote_data = func()
            if not ok:
                print("could not retrieve {}: {}".format(name, remote_data))
                continue

            local_data = json.load(file)
            equal = _are_jsons_equal(local_data, remote_data)
            if not equal:
                print("%s differ" % name)


@backup.command(name='restore', short_help="Restores all the information dumped from 'backup' to Monitor and Secure.")
@click.argument('backup-file', nargs=1)
@click.argument('section', nargs=-1)
@click.option('--full', is_flag=True, help="Drop and restore everything from the sections specified. "
                                           "If not specified only items that are different in the backup than in the remote will be restored.")
@click.option('--all-users', is_flag=True, help="Restore dashboards even if you are not the owner "
                                                "(may duplicate the dashboards if they already exist in the environment)")
@click.pass_obj
def restore_from_backup(cnf, backup_file, section, full, all_users):
    """
    BACKUP_PATH: File where the data must be restored from.

    SECTION: Restore only the specified section. Valid values: dashboards, notification_channels, alerts, users, teams_monitor, teams_secure, policies, falco_rules, falco_lists, falco_macros.
    """
    if not os.path.isfile(backup_file):
        print("Error: {} is not a correct backup file".format(backup_file))
        sys.exit(1)

    use_cases_to_execute = []

    if not section or 'dashboards' in section:
        if full:
            if all_users:
                use_cases_to_execute.append(RemoveAllAndRestoreAllDashboardsUseCase(cnf.sdmonitor, open(backup_file, mode='r')))
            else:
                use_cases_to_execute.append(RemoveOwnedAndRestoreOwnedDashboardsUseCase(cnf.sdmonitor, open(backup_file, mode='r')))
        else:
            if all_users:
                use_cases_to_execute.append(RestoreAllDashboardsUseCase(cnf.sdmonitor, open(backup_file, mode='r')))
            else:
                use_cases_to_execute.append(RestoreOwnedDashboardsUseCase(cnf.sdmonitor, open(backup_file, mode='r')))

    if not section or 'notification_channels' in section:
        if full:
            use_cases_to_execute.append(RemoveAndRestoreNotificationChannelsUseCase(cnf.sdmonitor, open(backup_file, mode='r')))
        else:
            use_cases_to_execute.append(RestoreNotificationChannelsUseCase(cnf.sdmonitor, open(backup_file, mode='r')))

    if not section or 'alerts' in section:
        if full:
            use_cases_to_execute.append(RemoveAndRestoreAlertsUseCase(cnf.sdmonitor, open(backup_file, mode='r')))
        else:
            use_cases_to_execute.append(RestoreAlertsUseCase(cnf.sdmonitor, open(backup_file, mode='r')))

    if not section or 'users' in section:
        use_cases_to_execute.append(RestoreUsersUseCase(cnf.sdmonitor, open(backup_file, mode='r')))

    if not section or 'teams_monitor' in section:
        if full:
            use_cases_to_execute.append(RemoveAndRestoreTeamsMonitorUseCase(cnf.sdmonitor, open(backup_file, mode='r')))
        else:
            use_cases_to_execute.append(RestoreTeamsMonitorUseCase(cnf.sdmonitor, open(backup_file, mode='r')))

    if not section or 'teams_secure' in section:
        if full:
            use_cases_to_execute.append(RemoveAndRestoreTeamsSecureUseCase(cnf.sdsecure, open(backup_file, mode='r')))
        else:
            use_cases_to_execute.append(RestoreTeamsSecureUseCase(cnf.sdsecure, open(backup_file, mode='r')))

    if full:
        if not section or 'policies' in section:
            use_cases_to_execute.append(RemovePoliciesUseCase(cnf.sdsecure))
        if not section or 'falco_rules' in section:
            use_cases_to_execute.append(RemoveFalcoRulesUseCase(cnf.sdsecure))
        if not section or 'falco_macros' in section:
            use_cases_to_execute.append(RemoveFalcoMacrosUseCase(cnf.sdsecure))
        if not section or 'falco_lists' in section:
            use_cases_to_execute.append(RemoveFalcoListsUseCase(cnf.sdsecure))

    if not section or 'falco_lists' in section:
        use_cases_to_execute.append(RestoreFalcoListsUseCase(cnf.sdsecure, open(backup_file, mode='r')))
    if not section or 'falco_macros' in section:
        use_cases_to_execute.append(RestoreFalcoMacrosUseCase(cnf.sdsecure, open(backup_file, mode='r')))
    if not section or 'falco_rules' in section:
        use_cases_to_execute.append(RestoreFalcoRulesUseCase(cnf.sdsecure, open(backup_file, mode='r')))
    if not section or 'policies' in section:
        use_cases_to_execute.append(RestorePoliciesUseCase(cnf.sdsecure, open(backup_file, mode='r')))

    errors = []
    for use_case in use_cases_to_execute:
        ok, res = use_case.execute()
        if not ok:
            errors.append(res)

    if len(errors) > 0:
        for error in errors:
            print(error)
        sys.exit(1)
    sys.exit(0)


def _are_jsons_equal(json1, json2):
    def ordered(o):
        if isinstance(o, dict):
            return sorted((k, ordered(v)) for k, v in o.items())
        if isinstance(o, list):
            return sorted(ordered(x) for x in o)
        return o

    return ordered(json1) == ordered(json2)



import json
import sys

import click

from sdccli.cli.dashboard.panel import panel

import sdccli.usecases.dashboard.dashboard_v3 as use_case
import sdccli.usecases.settings.team as team_usecase


@click.group(name='dashboard', short_help='Sysdig Monitor dashboard operations')
def dashboard():
    pass


dashboard.add_command(panel)


@dashboard.command(name='add', short_help="Add a dashboard")
@click.argument('name', nargs=1)
@click.argument('templatefile', nargs=1, type=click.Path(exists=True))
@click.option('--scope', nargs=1, multiple=True,
              help="filter to apply to the dashboard; must be based on metadata available in Sysdig Monitor; Example: kubernetes.namespace.name in [prod, dev]")
@click.option('--public', is_flag=True, help="if set the new dashboard will be shared with public token.")
@click.pass_obj
def add(cnf, name, templatefile, scope, public):
    """
    NAME: name for the new dashboard

    TEMPLATEFILE: A file with a json with "layout" and "items" elements
    """
    try:
        with open(templatefile) as f:
            template = json.load(f)
    except Exception as error:
        print("Error parsing rules file (%s): %s" % (templatefile, str(error)))
        sys.exit(1)

    try:
        res = use_case.add_dashboard(cnf.sdmonitor, name, template, list(scope), public)
        cnf.formatter.format(res, "dashboard")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@dashboard.command(name='add-json', short_help="Add dashboards from a json file")
@click.argument('jsonfile', nargs=1, type=click.Path(exists=True))
@click.option('--ignore-sharing-settings', is_flag=True, help="Ignores the sharing settings configured in the JSON, "
                                                              "not sending them to the API")
@click.pass_obj
def add_json(cnf, jsonfile, ignore_sharing_settings):
    """
    JSONFILE: A file with a json description of one or multiple dashboards
    """
    try:
        with open(jsonfile) as f:
            dashboard_json = json.load(f)
    except Exception as error:
        print("Error parsing rules file (%s): %s" % (jsonfile, str(error)))
        sys.exit(1)

    try:
        res = use_case.add_json_dashboard(cnf.sdmonitor, dashboard_json, ignore_sharing_settings)
        cnf.formatter.format(res, "dashboard")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)

@dashboard.command(name='update-json', short_help="Update dashboards from a json file")
@click.argument('jsonfile', nargs=1, type=click.Path(exists=True))
@click.option('--ignore-sharing-settings', is_flag=True, help="Ignores the sharing settings configured in the JSON, "
                                                              "not sending them to the API")
@click.pass_obj
def update_json(cnf, jsonfile, ignore_sharing_settings):
    """
    JSONFILE: A file with a json description of one or multiple dashboards
    """
    try:
        with open(jsonfile) as f:
            dashboard_json = json.load(f)
    except Exception as error:
        print("Error parsing rules file (%s): %s" % (jsonfile, str(error)))
        sys.exit(1)

    try:
        res = use_case.update_json_dashboard(cnf.sdmonitor, dashboard_json, ignore_sharing_settings)
        cnf.formatter.format(res, "dashboard")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)

@dashboard.command(name='list', short_help="List all dashboards")
@click.pass_obj
def list_dashboards(cnf):
    try:
        res = use_case.list_dashboards(cnf.sdmonitor)
        cnf.formatter.format(res, "dashboardList")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@dashboard.command(name='get', short_help="Get dashboard")
@click.argument('dashboard', nargs=1)
@click.pass_obj
def get(cnf, dashboard):
    """
    DASHBOARD: The dashboard (id or name) to get
    """
    try:
        res = use_case.get_dashboard_by_id_or_name(cnf.sdmonitor, dashboard)
        cnf.formatter.format(res, "dashboard")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@dashboard.command(name='del', short_help="Delete dashboards")
@click.argument('dashboards', nargs=-1, required=True)
@click.pass_obj
def delete(cnf, dashboards):
    """
    DASHBOARDS: Dashboard id to delete
    """
    try:
        use_case.delete_dashboards_by_id(cnf.sdmonitor, dashboards)
        print("Success")
    except Exception as ex:
        print(f"Error deleting dashboard: {str(ex)}")


@dashboard.command(name='share', short_help="Share a dashboard")
@click.argument('dashboards', nargs=-1, required=True)
@click.option('--all-teams', is_flag=True, help="Share dashboard with all my teams")
@click.option('--team', nargs=1, help="Share dashboard with the selected team (name or ID)")
@click.option('-w', is_flag=True, help="Share the dashboard as read-write for the selected team(s). "
                                       "If not specified, it will be shared as read-only.")
@click.pass_obj
def share(cnf, dashboards, all_teams, team, w):
    errors = []
    mode = "w" if w else "r"
    for dboard in dashboards:
        try:
            dboard = use_case.get_dashboard_by_id_or_name(cnf.sdmonitor, dboard)
        except Exception as ex:
            errors.append(ex)
            continue

        if all_teams:
            try:
                res = use_case.share_dashboard_with_all_teams(cnf.sdmonitor, dboard, mode)
                print(f"Shared {dboard['name']} with all teams")
            except Exception as ex:
                print(f"Could not share {dboard} with all teams")
                errors.append(ex.message)
        else:
            try:
                if team is None or team == "":
                    errors.append("Team not provided, specify one with --team or all with --all-teams")
                    break
                team = team_usecase.retrieve_team_by_name_or_id(cnf.sdmonitor, team)
                res = use_case.share_dashboard_with_team(cnf.sdmonitor, dboard, team["id"], mode)
                print(f"Shared {dboard['name']} with team {team['name']}")
            except Exception as ex:
                errors.append(ex.message)

    for error in errors:
        print(error)
    if len(errors) > 0:
        sys.exit(1)


@dashboard.command(name='unshare', short_help="Unshare a dashboard")
@click.argument('dashboards', nargs=-1, required=True)
@click.pass_obj
def share(cnf, dashboards):
    errors = []
    for dboard in dashboards:
        try:
            dboard = use_case.get_dashboard_by_id_or_name(cnf.sdmonitor, dboard)
        except Exception as ex:
            errors.append(ex)
            continue

        try:
            use_case.unshare_dashboard(cnf.sdmonitor, dboard)
            print(f"Unshared {dboard['name']}")
        except Exception as ex:
            errors.append(ex.message)

    for error in errors:
        print(error)
    if len(errors) > 0:
        sys.exit(1)


@dashboard.command(name='fav', short_help="Favorite a dashboard")
@click.argument('dashboards', nargs=-1, required=True)
@click.pass_obj
def fav(cnf, dashboards):
    errors = []
    for dboard in dashboards:
        try:
            dboard = use_case.get_dashboard_by_id_or_name(cnf.sdmonitor, dboard)
        except Exception as ex:
            errors.append(ex)
            continue

        try:
            use_case.favorite_dashboard(cnf.sdmonitor, dboard, True)
            print(f"{dboard['name']} is now favorite")
        except Exception as ex:
            errors.append(ex.message)

    for error in errors:
        print(error)
    if len(errors) > 0:
        sys.exit(1)


@dashboard.command(name='unfav', short_help="Unfavorite a dashboard")
@click.argument('dashboards', nargs=-1, required=True)
@click.pass_obj
def unfav(cnf, dashboards):
    errors = []
    for dboard in dashboards:
        try:
            dboard = use_case.get_dashboard_by_id_or_name(cnf.sdmonitor, dboard)
        except Exception as ex:
            errors.append(ex)
            continue

        try:
            use_case.favorite_dashboard(cnf.sdmonitor, dboard, False)
            print(f"{dboard['name']} is now not favorite")
        except Exception as ex:
            errors.append(ex.message)

    for error in errors:
        print(error)
    if len(errors) > 0:
        sys.exit(1)


from abc import abstractmethod, ABC
from multiprocessing.pool import ThreadPool

import yaml

ORIGIN_SECURE_UI = 'Secure UI'
ORIGIN_SYSDIG = 'Sysdig'


class DumpUseCase(ABC):
    @abstractmethod
    def execute(self):
        pass

    @staticmethod
    def _sort_res(res):
        if isinstance(res, list):
            res.sort(key=lambda item: item["id"])

        elif isinstance(res, dict):
            for k, v in res.items():
                if isinstance(v, list) and len(v) > 0 and "id" in v[0]:
                    res[k].sort(key=lambda item: item["id"])


class DumpMonitorUseCase(DumpUseCase):
    def __init__(self, monitor, repository, formatter=yaml.dump):
        self._monitor = monitor
        self._repository = repository
        self._formatter = formatter


class DumpSecureUseCase(DumpUseCase):
    def __init__(self, secure, repository, formatter=yaml.dump):
        self._secure = secure
        self._repository = repository
        self._formatter = formatter


class DumpUsersUseCase(DumpMonitorUseCase):
    def __init__(self, monitor, repository, formatter=yaml.dump):
        super().__init__(monitor, repository, formatter)

    def execute(self):
        ok, res = self._monitor.get_users()
        if not ok:
            return ok, res
        self._sort_res(res)

        res = [{"kind": "users", "version": 1, "spec": res}]

        self._repository.write(self._formatter(res))
        return ok, res


class DumpDashboardsUseCase(DumpMonitorUseCase):
    def __init__(self, monitor, repository):
        super().__init__(monitor, repository)

    def execute(self):
        ok, res = self._dashboard_getter()
        if not ok:
            return ok, res

        self._sort_res(res)

        res = [{"kind": "dashboards", "version": 1, "spec": res}]

        self._repository.write(self._formatter(res))
        return ok, res

    def _dashboard_getter(self):
        ok, res = self._monitor.get_dashboards()
        if not ok:
            return ok, res

        ok, teams = self._monitor.get_teams()
        team_match = {team["id"]: team["name"] for team in teams} if ok else {}
        dashboards = []
        errors = []
        for ok, data in ThreadPool().map(self._monitor.get_dashboard, (db["id"] for db in res["dashboards"])):
            if ok:
                data["dashboard"]["teamName"] = team_match.get(data["dashboard"].get("teamId", 0))
                dashboards.append(data["dashboard"])
            else:
                errors.append(data)

        if len(errors) > 0:
            return False, errors

        return True, {"dashboards": dashboards}


class DumpAlertsUseCase(DumpMonitorUseCase):
    def __init__(self, monitor, repository):
        super().__init__(monitor, repository)

    def execute(self):
        ok, res = self._monitor.get_alerts()
        if not ok:
            return ok, res
        self._sort_res(res)

        ok, resNC = self._monitor.list_notification_channels()
        if not ok:
            return ok, resNC


        res = [{"kind": "alerts", "version": 1, "spec": res}]

        self._repository.write(self._formatter(res))
        return ok, res


class DumpTeamsMonitorUseCase(DumpMonitorUseCase):
    def __init__(self, monitor, repository):
        super().__init__(monitor, repository)

    def execute(self):
        ok, res = self._monitor.get_teams()
        if not ok:
            return ok, res
        self._sort_res(res)

        res = [{"kind": "monitor_teams", "version": 1, "spec": res}]

        self._repository.write(self._formatter(res))
        return ok, res


class DumpNotificationChannelsUseCase(DumpMonitorUseCase):
    def __init__(self, monitor, repository):
        super().__init__(monitor, repository)

    def execute(self):
        ok, res = self._monitor.list_notification_channels()
        if not ok:
            return ok, res
        self._sort_res(res)

        res = [{"kind": "notification_channels", "version": 1, "spec": res}]

        self._repository.write(self._formatter(res))
        return ok, res


class DumpTeamsSecureUseCase(DumpSecureUseCase):
    def __init__(self, secure, repository):
        super().__init__(secure, repository)

    def execute(self):
        ok, res = self._secure.get_teams()
        if not ok:
            return ok, res
        self._sort_res(res)

        res = [{"kind": "secure_teams", "version": 1, "spec": res}]

        self._repository.write(self._formatter(res))
        return ok, res


class DumpPoliciesUseCase(DumpSecureUseCase):
    def __init__(self, secure, repository):
        super().__init__(secure, repository)

    def execute(self):
        ok, res = self._secure.list_policies()
        if not ok:
            return ok, res
        self._sort_res(res)
        # Restorable policies are the ones created by the Secure UI and the ones provided by Sysdig.
        # This only happens with policies, with other resources only the ones created by Secure UI can be removed.
        restorable_policies = [policy for policy in res if policy['origin'] in [ORIGIN_SECURE_UI, ORIGIN_SYSDIG]]

        res = [{"kind": "policies", "version": 1, "spec": restorable_policies}]

        self._repository.write(self._formatter(res))
        return ok, res


class DumpUserCreatedRulesUseCase(DumpSecureUseCase):
    def __init__(self, secure, repository):
        super().__init__(secure, repository)

    def execute(self):
        ok, res = self._rules_getter()
        if not ok:
            return ok, res
        self._sort_res(res)

        res = [{"kind": "falco_rules", "version": 1, "spec": res}]

        self._repository.write(self._formatter(res))
        return ok, res

    def _rules_getter(self):
        ok, res = self._get_user_created_rules()
        if not ok:
            return ok, res

        rules = []
        errors = []
        for ok, data in ThreadPool().map(self._secure.get_rule_id, (i["id"] for i in res)):
            if ok:
                rules.append(data)
            else:
                errors.append(data)

        if len(errors) > 0:
            return False, errors

        return True, rules

    # TODO: Move this to the python-sdc-client
    def _get_user_created_rules(self):
        ok, res = self._secure.list_rules()
        if not ok:
            return False, res

        user_created_rules = []
        for list in res:
            for j, id in enumerate(list['ids']):
                if list['publishedBys'][j]['origin'] == ORIGIN_SECURE_UI:
                    user_rules = list.copy()

                    user_rules['id'] = list['ids'][j]
                    del user_rules['ids']

                    user_rules['publishedBy'] = list['publishedBys'][j]
                    del user_rules['publishedBys']

                    user_created_rules.append(user_rules)

        return True, user_created_rules


class DumpUserCreatedFalcoMacrosUseCase(DumpSecureUseCase):
    def __init__(self, secure, repository):
        super().__init__(secure, repository)

    def execute(self):
        ok, res = self._falco_macros_getter()
        if not ok:
            return ok, res
        self._sort_res(res)

        res = [{"kind": "falco_macros", "version": 1, "spec": res}]

        self._repository.write(self._formatter(res))
        return ok, res

    # TODO: Move this to the python-sdc-cli
    def _get_user_created_macros(self):
        ok, res = self._secure.list_falco_macros()
        if not ok:
            return False, res

        user_created_macros = []
        for macro in res:
            for j, id in enumerate(macro['ids']):
                if macro['publishedBys'][j]['origin'] == ORIGIN_SECURE_UI:
                    user_macro = macro.copy()

                    user_macro['id'] = macro['ids'][j]
                    del user_macro['ids']

                    user_macro['publishedBy'] = macro['publishedBys'][j]
                    del user_macro['publishedBys']

                    user_created_macros.append(user_macro)

        return True, user_created_macros

    def _falco_macros_getter(self):

        ok, res = self._get_user_created_macros()
        if not ok:
            return ok, res

        falco_macros = []
        errors = []
        for ok, data in ThreadPool().map(self._secure.get_falco_macro_id, (i['id'] for i in res)):
            if ok:
                falco_macros.append(data)
            else:
                errors.append(data)

        if len(errors) > 0:
            return False, errors
        return True, falco_macros


class DumpUserCreatedFalcoListsUseCase(DumpSecureUseCase):
    def __init__(self, secure, repository):
        super().__init__(secure, repository)

    def execute(self):
        ok, res = self._falco_lists_getter()
        if not ok:
            return ok, res
        self._sort_res(res)

        res = [{"kind": "falco_lists", "version": 1, "spec": res}]

        self._repository.write(self._formatter(res))
        return ok, res

    # TODO: Move this to the python-sdc-client
    def _get_user_created_lists(self):
        ok, res = self._secure.list_falco_lists()
        if not ok:
            return False, res

        user_created_lists = []
        for list in res:
            for j, id in enumerate(list['ids']):
                if list['publishedBys'][j]['origin'] == ORIGIN_SECURE_UI:
                    user_list = list.copy()

                    user_list['id'] = list['ids'][j]
                    del user_list['ids']

                    user_list['publishedBy'] = list['publishedBys'][j]
                    del user_list['publishedBys']

                    user_created_lists.append(user_list)

        return True, user_created_lists

    def _falco_lists_getter(self):
        ok, res = self._get_user_created_lists()
        if not ok:
            return ok, res

        falco_lists = []
        errors = []
        for ok, data in ThreadPool().map(self._secure.get_falco_list_id, (i["id"] for i in res)):
            if ok:
                falco_lists.append(data)
            else:
                errors.append(data)

        if len(errors) > 0:
            return False, errors

        return True, falco_lists


class DumpEverythingUseCase(object):
    def __init__(self, monitor, secure, repository):
        # Monitor
        self._dump_users_to_repository = DumpUsersUseCase(monitor, repository)
        self._dump_dashboards_to_repository = DumpDashboardsUseCase(monitor, repository)
        self._dump_alerts_to_repository = DumpAlertsUseCase(monitor, repository)
        self._dump_teams_monitor_to_repository = DumpTeamsMonitorUseCase(monitor, repository)
        self._dump_notification_channels_to_repository = DumpNotificationChannelsUseCase(monitor,
                                                                                         repository)
        # Secure
        self._dump_teams_secure_to_repository = DumpTeamsSecureUseCase(secure, repository)
        self._dump_policies_to_repository = DumpPoliciesUseCase(secure, repository)
        self._dump_rules_to_repository = DumpUserCreatedRulesUseCase(secure, repository)
        self._dump_falco_macros_to_repository = DumpUserCreatedFalcoMacrosUseCase(secure, repository)
        self._dump_falco_lists_to_repository = DumpUserCreatedFalcoListsUseCase(secure, repository)

    def execute(self):
        errors = []

        # Monitor
        ok, res = self._dump_users_to_repository.execute()
        if not ok:
            errors.append(res)

        ok, res = self._dump_dashboards_to_repository.execute()
        if not ok:
            errors.append(res)

        ok, res = self._dump_alerts_to_repository.execute()
        if not ok:
            errors.append(res)

        ok, res = self._dump_teams_monitor_to_repository.execute()
        if not ok:
            errors.append(res)

        ok, res = self._dump_notification_channels_to_repository.execute()
        if not ok:
            errors.append(res)

        # Secure
        ok, res = self._dump_teams_secure_to_repository.execute()
        if not ok:
            errors.append(res)

        ok, res = self._dump_policies_to_repository.execute()
        if not ok:
            errors.append(res)

        ok, res = self._dump_rules_to_repository.execute()
        if not ok:
            errors.append(res)

        ok, res = self._dump_falco_lists_to_repository.execute()
        if not ok:
            errors.append(res)

        if len(errors) > 0:
            return False, errors

        return True, []



import click
import json
import sys
from prettytable import PrettyTable, PLAIN_COLUMNS


@click.group(short_help='Query operations')
def query():
    pass


@query.command(name='images-by-vulnerability', short_help="Search system for images with the given vulnerability ID present")
@click.argument('vulnerability_id', nargs=1)
@click.option('--namespace', help="Filter results to images with vulnerable packages in the given namespace (e.g. debian:9)")
@click.option('--package', help="Filter results to images with the given vulnerable package name (e.g. sed)")
@click.option('--severity', help="Filter results to images with the given vulnerability severity (e.g. Medium)")
@click.option('--vendor_only', is_flag=True, help="Only show images with vulnerabilities explicitly deemed applicable by upstream OS vendor, if present")
@click.pass_obj
def images_by_vulnerability(cnf, vulnerability_id, namespace, package, severity, vendor_only):
    """
    VULNERABILITY_ID: Search for images vulnerable to this vulnerability ID (e.g. CVE-1999-0001)
    """
    ok, res = cnf.sdscanning.query_images_by_vulnerability(
        vulnerability_id,
        namespace=namespace,
        package=package,
        severity=severity,
        vendor_only=vendor_only)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        _print_vulns(res['images'])
    else:
        print("Error: " + str(res))
        sys.exit(1)


@query.command(name='images-by-package', short_help="Search system for images with the given package installed")
@click.argument('name', nargs=1)
@click.option('--version', help="Filter results to only packages with given version (e.g. 4.4-1)")
@click.option('--package-type', help="Filter results to only packages of given type (e.g. dpkg)")
@click.pass_obj
def images_by_package(cnf, name, version, package_type):
    """
    NAME: Search for images with this package name (e.g. sed)
    """
    ok, res = cnf.sdscanning.query_images_by_package(
        name,
        version=version,
        package_type=package_type)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        _print_package(res['images'])
    else:
        print("Error: " + str(res))
        sys.exit(1)


def _print_vulns(images):
    header = ['Full Tag', 'Severity', 'Package', 'Package Type', 'Namespace', 'Digest']
    t = PrettyTable(header)
    t.set_style(PLAIN_COLUMNS)
    t.align = 'l'

    for image in images:
        for tag in image.get('image', {}).get('tag_history', []):
            for package in image.get('vulnerable_packages', []):
                row = [
                    tag.get('fulltag', "N/A"),
                    package.get('severity', "N/A"),
                    "{}-{}".format(package.get("name"), package.get("version")),
                    package.get("type"),
                    package.get('namespace', "N/A"),
                    image.get('image', {}).get('imageDigest', "N/A")]
                t.add_row(row)
    print(t.get_string(sortby='Full Tag'))


def _print_package(images):
    header = ['Full Tag', 'Package', 'Package Type', 'Digest']
    t = PrettyTable(header)
    t.set_style(PLAIN_COLUMNS)
    t.align = 'l'

    for image in images:
        for tag in image.get('image', {}).get('tag_history', []):
            for package in image.get('packages', []):
                row = [
                    tag.get('fulltag', "N/A"),
                    "{}-{}".format(package.get("name"), package.get("version")),
                    package.get("type"),
                    image.get('image', {}).get('imageDigest', "N/A")]
                t.add_row(row)
    print(t.get_string(sortby='Full Tag'))



import click
from sdccli.cli.settings.notification import notification
from sdccli.cli.settings.user import user
from sdccli.cli.settings.team import team
from sdccli.cli.settings.access_key import access_key


@click.group(short_help='Settings operations')
@click.option('--secure', is_flag=True, help='Work with Sysdig Secure settings instead of Sysdig Monitor')
@click.pass_obj
def settings(cnf, secure):
    cnf.sd = cnf.sdsecure if secure else cnf.sdmonitor


settings.add_command(notification)
settings.add_command(user)
settings.add_command(team)
settings.add_command(access_key)



import sys
from threading import Thread
from time import sleep

import click

import sdccli.usecases.scanning.vulnerability as use_case
from sdccli.cli.scanning.vulnerability.bundle import bundle


@click.group(name='vulnerability', short_help='Vulnerability operations')
def vulnerability():
    pass


@vulnerability.command(name='report', short_help="Downloads a vulnerability report in CSV format")
@click.option("--runtime", help="Generate vulnerability report for runtime images only", type=bool, is_flag=True)
@click.option("--os",
              help="Generates a vulnerability report for OS images only. By default generates for OS and NonOS.",
              type=bool,
              is_flag=True)
@click.option("--non-os", help="Generates a vulnerability report for NonOS images (pip, npm, java, ...) only. "
                               "By default generates for OS and NonOS.", type=bool, is_flag=True)
@click.argument('report_path', nargs=1)
@click.pass_obj
def report(cnf, runtime, os, non_os, report_path):
    """
    Downloads a vulnerability report at the given path.
    """
    if cnf.json:
        print("Json format is not supported for this method")
        sys.exit(1)

    # Prints a message after 2 minutes waiting for the generation of the report
    def wait_time():
        sleep(120)
        print("It looks like the report is taking more time than expected.")
        print("This may be due to the high amount of images in your system.")

    try:
        with open(report_path, "w") as f:
            print(f"Downloading report at {f.name}.")
            Thread(target=wait_time, daemon=True).run()
            use_case.download_vulnerability_report(cnf.sdscanning, runtime, os, non_os, f)
            print("Report downloaded")
    except Exception as ex:
        if "504" in str(ex):
            print("Timed out while waiting for the CVE report to be downloaded. "
                  "Max time limit is 3 minutes to retrieve either OS or NonOS reports.")
            if not os and not non_os:
                print("You can try executing again with --os or --non-os options for a fewer number of images.")
            if not runtime:
                print("You can also try using the --runtime options which only downloads a report or running images.")
            sys.exit(1)
        print(ex)
        sys.exit(1)


@vulnerability.command(name='get_info', short_help="Retrieves information about a vulnerability")
@click.argument('cve', nargs=1)
@click.pass_obj
def add(cnf, cve):
    """
    CVE: The CVE from which more info should be retrieved.
    """
    try:
        res = use_case.get_vulnerability_details(cnf.sdscanning, cve)
        cnf.formatter.format(res, "vulnerabilityDetails")
    except Exception as ex:
        print(ex)


@vulnerability.command(name='add_exception', short_help="Adds a vulnerability exception to a bundle")
@click.argument('bundleid', nargs=1)
@click.argument('cve', nargs=1)
@click.option('--notes', nargs=1, help="Notes for the vulnerability exception")
@click.option('--expiration-date', nargs=1, help="Expiration date in Unix time format")
@click.pass_obj
def add(cnf, bundleid, cve, notes=None, expiration_date=None):
    """
    BUNDLEID: The Bundle ID to add the CVE exception to.
    CVE: The CVE that should be added
    """
    try:
        res = use_case.add_vulnerability_exception(cnf.sdscanning, bundleid, cve, notes, expiration_date)
        cnf.formatter.format(res, "vulnerabilityExceptionAdded")

    except Exception as ex:
        print(ex)


@vulnerability.command(name='del_exception', short_help="Deletes a vulnerability exception from a bundle")
@click.argument('bundleid', nargs=1)
@click.argument('cve', nargs=1)
@click.pass_obj
def delete(cnf, bundleid, cve):
    """
    BUNDLEID: The Bundle ID to remove the CVE exception from.
    CVE: The CVE that should be deleted
    """
    try:
        res = use_case.delete_vulnerability_exception(cnf.sdscanning, bundleid, cve)
        print("Succeed")

    except Exception as ex:
        print(ex)


vulnerability.add_command(bundle)



import click
import json
import sys
from sdccli.printer import print_list


_list_keys = ["subscription_key", "subscription_type", "active"]


@click.group(name='subscription', short_help='Subscription operations')
def subscription():
    pass


@subscription.command(name='activate', short_help="Activate a subscription")
@click.argument('subscription_type', nargs=1)
@click.argument('subscription_key', nargs=1, required=False)
@click.pass_obj
def activate(cnf, subscription_type, subscription_key):
    """
    SUBSCRIPTION_TYPE: Type of subscription. Valid options:

      - tag_update: Receive notification when new image is pushed

      - policy_eval: Receive notification when image policy status changes

      - vuln_update: Receive notification when vulnerabilities are added, removed or modified

    SUBSCRIPTION_KEY: Fully qualified name of tag to subscribe to. Eg. docker.io/library/alpine:latest
    """
    ok, res = cnf.sdscanning.activate_subscription(subscription_type, subscription_key)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print("Success")
    else:
        print("Error: " + str(res))
        sys.exit(1)


@subscription.command(name='list', short_help="List all current subscriptions")
@click.pass_obj
def list(cnf):
    ok, res = cnf.sdscanning.list_subscription()

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_list(res, _list_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@subscription.command(name='deactivate', short_help="Deactivate a subscription")
@click.argument('subscription_type', nargs=1)
@click.argument('subscription_key', nargs=1, required=False)
@click.pass_obj
def deactivate(cnf, subscription_type, subscription_key):
    """
    SUBSCRIPTION_TYPE: Type of subscription. Valid options:

      - tag_update: Receive notification when new image is pushed

      - policy_eval: Receive notification when image policy status changes

      - vuln_update: Receive notification when vulnerabilities are added, removed or modified

    SUBSCRIPTION_KEY: Fully qualified name of tag to subscribe to. Eg. docker.io/library/alpine:latest
    """
    ok, res = cnf.sdscanning.deactivate_subscription(subscription_type, subscription_key)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print("Success")
    else:
        print("Error: " + str(res))
        sys.exit(1)



import dateutil.parser
from sdcclient import SdScanningClient


def add_scanning_image(scanning: SdScanningClient, input_image, force, dockerfile, annotations, autosubscribe):
    ok, res = scanning.add_image(
        input_image,
        force=force,
        dockerfile=dockerfile,
        annotations=annotations,
        autosubscribe=autosubscribe)

    if not ok:
        raise Exception(res)

    return res


def list_scanning_images(scanning: SdScanningClient, show_all=True):
    ok, images = scanning.list_images()
    if not ok:
        raise Exception(images)

    if not show_all:
        images = _get_latest_image_records(images)
    return images


def _get_latest_image_records(images):
    latest_tag_details = {}
    latest_records = {}
    for image_record in images:
        for image_detail in image_record['image_detail']:
            fulltag = image_detail['fulltag']
            tagts = dateutil.parser.parse(image_detail['created_at'])
            if fulltag not in latest_tag_details:
                latest_tag_details[fulltag] = image_detail
                latest_records[fulltag] = image_record
            else:
                lasttagts = dateutil.parser.parse(latest_tag_details[fulltag]['created_at'])
                if tagts >= lasttagts:
                    latest_tag_details[fulltag] = image_detail
                    latest_records[fulltag] = image_record

    return list(latest_records.values())


def get_image_from_digest_id_or_repo(scanning: SdScanningClient, input_image, show_history):
    ok, res = scanning.get_image(input_image, show_history)
    if not ok:
        raise Exception(res)

    return res[0]


def query_image_content(scanning: SdScanningClient, input_image, content_type):
    supported_content_types = ["os", "npm", "gem", "files", "java", "python", "all", "all+files"]
    content_type = content_type.lower()

    if content_type not in supported_content_types:
        raise ValueError(f"Incorrect content type provided '{content_type}', must be one of: {supported_content_types}")

    if content_type not in ["all", "all+files"]:
        ok, res = scanning.query_image_content(input_image, content_type)
        if not ok:
            raise Exception(res)

        return res

    if content_type == "all":
        result = {"content": [], "content_type": "all"}

        for type in ["os", "npm", "gem", "java", "python"]:
            result["content"].append(query_image_content(scanning, input_image, type))

        return result

    if content_type == "all+files":
        result = {"content": [], "content_type": "all+files"}

        for type in ["os", "npm", "gem", "java", "python", "files"]:
            result["content"].append(query_image_content(scanning, input_image, type))

        return result

    raise Exception("unreachable code reached! report this to the CLI maintainers")


def query_image_metadata(scanning: SdScanningClient, input_image, metadata_type):
    if metadata_type not in ["manifest", "dockerfile", "docker_history"]:
        raise ValueError(
            "Incorrect metadata type provided 'foo', must be one of: [manifest, dockerfile, docker_history]")
    ok, res = scanning.query_image_metadata(input_image, metadata_type)


def query_image_vuln(scanning: SdScanningClient, input_image, vuln_type="all", vendor_only=True):
    if not vuln_type in ["os", "non-os", "all"]:
        raise ValueError("Incorrect vulnerability type provided, must be one of: [os, non-os, all]")

    ok, res = scanning.query_image_vuln(input_image, vuln_type, vendor_only=vendor_only)
    if not ok:
        raise Exception(res)

    return res


def get_image_scanning_results(scanning: SdScanningClient, input_image, policy):
    ok, res = scanning.get_image_scanning_results(input_image, policy)
    if not ok:
        raise Exception(res)
    return res


def get_pdf_report(scanning: SdScanningClient, input_image, tag, date):
    ok, res = scanning.get_pdf_report(
        input_image,
        tag=tag,
        date=date
    )
    if not ok:
        raise Exception(res)

    return res



import json
from prettytable import PrettyTable, PLAIN_COLUMNS


def print_list(lst, keys):
    t = PrettyTable(keys)
    t.set_style(PLAIN_COLUMNS)
    t.align = 'l'

    for item in lst:
        values = [item[key]
                  if key in item else ""
                  for key in keys]
        t.add_row(values)
    print(t.get_string())


def print_item(item, keys):
    for key in keys:
        if key in item:
            if (item[key] and (isinstance(item[key], list) and isinstance(item[key][0], dict))
                    or isinstance(item[key], dict)):
                print(key + ":")
                for line in json.dumps(item[key], indent=2).split('\n'):
                    print("    " + line)
            else:
                print("{:25} {}".format(key + ":", item[key]))



from sdcclient.monitor import DashboardsClientV3


class Panel:
    def __init__(self, name, type, scope, metrics, sort, layout_dict, limit):
        self.name = name
        self.type = type
        self.scope = scope
        self.metrics = metrics
        self.sort = sort
        self.layout_dict = layout_dict
        self.limit = limit


class PanelBuilder:
    def __init__(self):
        self._name = "Panel"
        self._type = "timeSeries"
        self._scope = None
        self._metrics = []
        self._layout_dict = None
        self._sort = "desc"
        self._limit = None

    def with_layout(self, row, col, width, height):
        self._layout_dict = {
            "row": row,
            "col": col,
            "size_x": width,
            "size_y": height
        }
        return self

    def with_metric(self, id, time=None, group=None):
        if not id:
            raise Exception(f"cannot build a panel without id")
        metric_dict = {'id': id}
        if time and group:
            metric_dict["aggregations"] = {}
            metric_dict["aggregations"]['time'] = time
            metric_dict["aggregations"]['group'] = group
        self._metrics.append(metric_dict)
        return self

    def with_name(self, name):
        self._name = name
        if not name or not isinstance(name, str):
            raise Exception(f"name invalid or empty: '{name}'")
        return self

    def with_type(self, type):
        valid_types = ['timeSeries', 'top', 'number']
        if type not in valid_types:
            raise Exception(f"invalid type {type}, must be one of {valid_types}")
        self._type = type
        return self

    def with_scope(self, scope):
        self._scope = scope
        return self

    def with_limit(self, limit):
        if limit and limit > 10:
            raise Exception(f"invalid limit {limit} higher than the max of 10, could cause rendering problems")
        self._limit = limit
        return self

    def with_sort(self, sort):
        self._sort = sort
        return self

    def build(self):
        return Panel(name=self._name,
                     type=self._type,
                     scope=self._scope,
                     metrics=self._metrics,
                     sort=self._sort,
                     layout_dict=self._layout_dict,
                     limit=self._limit)


def add_dashboard(monitor, name, template, scope, public):
    ok, res = monitor.create_dashboard_from_template(
        name, template, scope,
        public=public)

    if not ok:
        raise Exception(res)
    return res["dashboard"]


def list_dashboards(monitor):
    ok, res = monitor.get_dashboards()

    if not ok:
        raise Exception(res)

    return res["dashboards"]


def delete_dashboards_by_id(monitor, ids):
    for id in ids:
        ok, res = monitor.delete_dashboard({"id": id})
        if not ok:
            raise Exception(res)


def get_dashboard_by_id_or_name(monitor, dashboard):
    try:
        # By ID
        id = int(dashboard)
        ok, res = monitor.get_dashboards()
        if not ok:
            raise Exception(res)

        for d in res["dashboards"]:
            if id == d["id"]:
                ok, res = monitor.get_dashboard(d["id"])
                if not ok:
                    raise Exception(res)
                return res["dashboard"]

        raise Exception(f"no dashboard with id {id}")
    except ValueError:
        # By Name
        ok, res = monitor.find_dashboard_by(dashboard)
        if not ok:
            raise Exception(res)

        if len(res) == 0:
            raise Exception(f"no dashboard with name '{dashboard}' found")

        ok, res = monitor.get_dashboard(res[0]["dashboard"]["id"])
        if not ok:
            raise Exception(res)
        return res["dashboard"]


def add_panel_to_dashboard(monitor, dashboard, panel):
    ok, res = monitor.add_dashboard_panel(
        dashboard, panel.name, panel.type, panel.metrics,
        scope=panel.scope,
        sort_direction=panel.sort,
        limit=panel.limit,
        layout=panel.layout_dict)
    if not ok:
        raise Exception(res)

    return res["dashboard"]


def add_json_dashboard(monitor, dashboard):
    result = ""

    def add_json(monitor, dboard):
        dboard['timeMode'] = {'mode': 1}
        dboard['time'] = {'last': 2 * 60 * 60 * 1000000, 'sampling': 2 * 60 * 60 * 1000000}
        ok, res = monitor.create_dashboard_with_configuration(dboard)
        if ok:
            return "Created dashboard {}".format(dboard['name'])
        else:
            raise Exception("Error creating the dashboard {}: {}".format(dboard['name'], res))

    if isinstance(dashboard, list):
        for db in dashboard:
            result += add_json(monitor, db) + "\n"
    elif isinstance(dashboard, dict):
        result = add_json(monitor, dashboard)
    else:
        raise Exception("Unsupported JSON format")

    return result


def remove_panel_from_dashboard(monitor, dashboard, name):
    ok, res = monitor.remove_dashboard_panel(dashboard, name)

    if not ok:
        raise Exception("Error deleting dashboard ({}): {}".format(id, res))
